#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :   __main__.py
@Time    :   2022/04/28 06:13:15
@Author  :   Calros Teng 
@Version :   1.0
@Contact :   303359166@qq.com
@License :   (C)Copyright 2017-2018, Xin Yuan Studio
@Desc    :   主启动文件
"""

# here put the import lib

# here put the import lib
# 设置环境变量切换成生产模式
import os

os.environ["MODE"] = "development"

import sys
from perfcat.app import PerfcatApplication
from perfcat import asset_rc


def main():
    app = PerfcatApplication(sys.argv)
    return app.exec()


sys.exit(main())



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'ui_left_column.ui'
##
## Created by: Qt User Interface Compiler version 6.3.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (
    QCoreApplication,
    QDate,
    QDateTime,
    QLocale,
    QMetaObject,
    QObject,
    QPoint,
    QRect,
    QSize,
    QTime,
    QUrl,
    Qt,
)
from PySide6.QtGui import (
    QBrush,
    QColor,
    QConicalGradient,
    QCursor,
    QFont,
    QFontDatabase,
    QGradient,
    QIcon,
    QImage,
    QKeySequence,
    QLinearGradient,
    QPainter,
    QPalette,
    QPixmap,
    QRadialGradient,
    QTransform,
)
from PySide6.QtWidgets import (
    QApplication,
    QFrame,
    QHBoxLayout,
    QLabel,
    QSizePolicy,
    QStackedWidget,
    QTextBrowser,
    QToolButton,
    QVBoxLayout,
    QWidget,
)
import asset_rc


class Ui_LeftColumn(object):
    def setupUi(self, LeftColumn):
        if not LeftColumn.objectName():
            LeftColumn.setObjectName("LeftColumn")
        LeftColumn.resize(240, 549)
        LeftColumn.setMaximumSize(QSize(240, 16777215))
        LeftColumn.setStyleSheet(
            "* {\n"
            "    border-radius:8;\n"
            "    color:#6c7c96;\n"
            "}\n"
            "\n"
            "\n"
            "#LeftColumn{\n"
            "background-color:#343b48;\n"
            "border-radius:8;\n"
            "}\n"
            "\n"
            "#LeftColumn #top{\n"
            "background-color: #3c4454;\n"
            "border-radius:8;\n"
            "}\n"
            "\n"
            "#LeftColumn #title{\n"
            "color:#6c7c96;\n"
            "}\n"
            "\n"
            "#LeftColumn QToolButton{\n"
            "border-radius:8;\n"
            "border:none;\n"
            "padding:4;\n"
            "}\n"
            "\n"
            "#LeftColumn #btn_close:hover{\n"
            "background-color:#343b48;\n"
            "}\n"
            "\n"
            "#LeftColumn #btn_close:pressed{\n"
            "background-color:#1b1e23;\n"
            "}\n"
            "\n"
            "\n"
            ""
        )
        self.verticalLayout = QVBoxLayout(LeftColumn)
        self.verticalLayout.setObjectName("verticalLayout")
        self.top = QFrame(LeftColumn)
        self.top.setObjectName("top")
        self.top.setMinimumSize(QSize(0, 32))
        self.top.setMaximumSize(QSize(16777215, 50))
        self.horizontalLayout = QHBoxLayout(self.top)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.horizontalLayout.setContentsMargins(4, 4, 4, 4)
        self.icon = QToolButton(self.top)
        self.icon.setObjectName("icon")
        self.icon.setEnabled(True)
        self.icon.setMinimumSize(QSize(32, 32))
        icon1 = QIcon()
        icon1.addFile(
            ":/icon_w/assets/svg_white/info.svg", QSize(), QIcon.Normal, QIcon.Off
        )
        self.icon.setIcon(icon1)
        self.icon.setAutoRaise(False)

        self.horizontalLayout.addWidget(self.icon)

        self.title = QLabel(self.top)
        self.title.setObjectName("title")

        self.horizontalLayout.addWidget(self.title)

        self.btn_close = QToolButton(self.top)
        self.btn_close.setObjectName("btn_close")
        self.btn_close.setMinimumSize(QSize(32, 32))
        self.btn_close.setMaximumSize(QSize(32, 32))
        icon2 = QIcon()
        icon2.addFile(
            ":/icon_w/assets/svg_white/cross.svg", QSize(), QIcon.Normal, QIcon.Off
        )
        icon2.addFile(
            ":/icon_b/assets/svg_blue/cross.svg", QSize(), QIcon.Normal, QIcon.On
        )
        self.btn_close.setIcon(icon2)

        self.horizontalLayout.addWidget(self.btn_close)

        self.verticalLayout.addWidget(self.top)

        self.stacked = QStackedWidget(LeftColumn)
        self.stacked.setObjectName("stacked")
        self.stacked.setMinimumSize(QSize(220, 0))
        self.page_setting = QWidget()
        self.page_setting.setObjectName("page_setting")
        self.verticalLayout_2 = QVBoxLayout(self.page_setting)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.stacked.addWidget(self.page_setting)
        self.page_about = QWidget()
        self.page_about.setObjectName("page_about")
        self.verticalLayout_3 = QVBoxLayout(self.page_about)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.verticalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.te_about = QTextBrowser(self.page_about)
        self.te_about.setObjectName("te_about")
        self.te_about.setStyleSheet(
            "QTextEdit{\n" "    background-color:#3c4454;\n" "	padding:8px;\n" "}\n" ""
        )
        self.te_about.setReadOnly(True)
        self.te_about.setAcceptRichText(True)

        self.verticalLayout_3.addWidget(self.te_about)

        self.stacked.addWidget(self.page_about)

        self.verticalLayout.addWidget(self.stacked)

        self.retranslateUi(LeftColumn)

        self.stacked.setCurrentIndex(1)

        QMetaObject.connectSlotsByName(LeftColumn)

    # setupUi

    def retranslateUi(self, LeftColumn):
        LeftColumn.setWindowTitle(
            QCoreApplication.translate("LeftColumn", "\u4fa7\u8fb9\u680f", None)
        )
        self.icon.setText(QCoreApplication.translate("LeftColumn", "PushButton", None))
        self.title.setText(
            QCoreApplication.translate("LeftColumn", "\u8bbe\u7f6e", None)
        )
        self.btn_close.setText("")
        self.te_about.setMarkdown(
            QCoreApplication.translate(
                "LeftColumn", "**Perfcat**\n" "\n" "v 1.0.1\n" "\n" "", None
            )
        )
        self.te_about.setHtml(
            QCoreApplication.translate(
                "LeftColumn",
                '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0//EN" "http://www.w3.org/TR/REC-html40/strict.dtd">\n'
                '<html><head><meta name="qrichtext" content="1" /><meta charset="utf-8" /><style type="text/css">\n'
                "p, li { white-space: pre-wrap; }\n"
                "hr { height: 1px; border-width: 0; }\n"
                "</style></head><body style=\" font-family:'Microsoft YaHei UI'; font-size:9pt; font-weight:400; font-style:normal;\">\n"
                '<p style=" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;"><span style=" font-size:18pt; font-weight:700;">Perfcat</span></p>\n'
                '<p style=" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;"><span style=" font-size:10pt;">v 1.0.1</span></p></body></html>',
                None,
            )
        )

    # retranslateUi



import logging
from typing import Any, Optional, Union, List
from PySide6.QtCore import (
    QAbstractTableModel,
    QObject,
    QModelIndex,
    QPersistentModelIndex,
    Qt,
    QSortFilterProxyModel,
)
from PySide6.QtGui import QColor
from perfcat.ui.constant import Color
from .log_provider import LogProvider

log = logging.getLogger(__name__)


class LogModel(QAbstractTableModel):
    COL_DATE = 0
    COL_TIME = 1
    COL_PID = 2
    COL_TID = 3
    COL_LEVEL = 4
    COL_TAG = 5
    COL_CONTENT = 6

    def __init__(self, parent: Optional[QObject] = None) -> None:
        super().__init__(parent)
        self.log_provider = LogProvider(self)
        self.log_provider.new_log_added.connect(self.on_new_log_added)
        self._logs = []
        self._headers = ["日期", "时间", "PID", "TID", "优先级", "标签", "消息"]

    def start(self, serial: str):
        self.log_provider.start(serial)

    def stop(self):
        self.log_provider.exit()

    def rowCount(self, parent: Union[QModelIndex, QPersistentModelIndex] = None) -> int:
        return len(self._logs)

    def on_new_log_added(self, msg: list):
        row = self.rowCount()
        length = len(msg)
        self.beginInsertRows(QModelIndex(), row, row + length - 1)
        self._logs += msg
        self.endInsertRows()

    def data(
        self, index: Union[QModelIndex, QPersistentModelIndex], role: int = ...
    ) -> Any:
        if role == Qt.DisplayRole:
            return self._logs[index.row()][index.column()]
        elif role == Qt.EditRole:  # 编辑时
            return self._logs[index.row()][index.column()]
        if role == Qt.ForegroundRole:
            level = self._logs[index.row()][self.COL_LEVEL]
            if "E" == level:
                return QColor(Color["danger"])
            elif "W" == level:
                return QColor(Color["warning"])
            elif "D" == level:
                return QColor(Color["success"])
            elif "I" == level:
                return QColor(Color["info"])

    def columnCount(
        self, parent: Union[QModelIndex, QPersistentModelIndex] = ...
    ) -> int:
        return len(self._headers)

    def headerData(
        self, section: int, orientation: Qt.Orientation, role: int = ...
    ) -> Any:
        if role == Qt.DisplayRole and orientation == Qt.Horizontal:
            return self._headers[section]

    def clear(self):
        self.beginResetModel()
        self._logs = []
        self.endResetModel()

    def flags(self, index: Union[QModelIndex, QPersistentModelIndex]) -> Qt.ItemFlags:
        return Qt.ItemIsEnabled | Qt.ItemIsEditable | Qt.ItemIsSelectable

    def get_text_by_indexes(self, indexes: List[QModelIndex]):
        lines = []
        row = None

        line = []
        for index in indexes:
            if row is None:
                row = index.row()
            elif index.row() != row:
                lines.append("  ".join(line))
                line.clear()
                row = index.row()
            line.append(index.data())

        lines.append("  ".join(line))

        return "\n".join(lines)

    def log_text(self):
        lines = []
        for each_row in self._logs:
            line = "  ".join(each_row) + "\n"
            lines.append(line)

        return lines


class LogSortFilterProxyModel(QSortFilterProxyModel):
    def __init__(self, parent=None):
        super(LogSortFilterProxyModel, self).__init__(parent)
        self.role = Qt.DisplayRole

        self.filter_map = {}  # 各列正在使用的正则表达式

    def filterAcceptsRow(self, sourceRow, sourceParent):
        """
        sourceRow : 行数
        sourceParent ：
        """
        for col, regex in self.filter_map.items():
            index = self.sourceModel().index(sourceRow, col, sourceParent)
            if index.isValid():
                text: str = self.sourceModel().data(index, Qt.DisplayRole)
                if self.filterCaseSensitivity() == Qt.CaseInsensitive:
                    if regex.lower() not in text.lower():
                        return False
                else:
                    if regex not in text:
                        return False
        return True

    def set_filter_by_column(self, pattern: str, column: int):
        self.filter_map[column] = pattern
        self.invalidateFilter()



from ppadb.device import Device
from .base.chart import MonitorChart
from PySide6.QtCharts import QLineSeries

class MemMonitor(MonitorChart):
    def __init__(self, parent=None):
        super().__init__(
            y_axis_name="MEM",
            parent=parent,
        )
        self.setObjectName("Memory")

        self._sample_data ={}

        self.create_series("PSS", QLineSeries(self),lambda v: f"{v}MB")
        self.create_series("PrivateDirty", QLineSeries(self),lambda v: f"{v}MB")
        self.create_series("PrivateClean", QLineSeries(self),lambda v: f"{v}MB")
        self.create_series("SwappedDirty", QLineSeries(self),lambda v: f"{v}MB")
        self.create_series("HeapSize", QLineSeries(self),lambda v: f"{v}MB")
        self.create_series("HeapAlloc", QLineSeries(self),lambda v: f"{v}MB")
        self.create_series("HeapFree", QLineSeries(self),lambda v: f"{v}MB")

    def reset_series_data(self):
        self._sample_data = {}
        return super().reset_series_data()


    def sample(self, sec: int, device: Device, package_name: str):

        mem_info = device.get_meminfo(package_name)

        self._sample_data[sec] = {
            "PSS":round(mem_info.pss / 1024,2),
            "PrivateDirty":round(mem_info.private_dirty / 1024,2),
            "PrivateClean": round(mem_info.private_clean/ 1024,2),
            "SwappedDirty": round(mem_info.swapped_dirty/ 1024,2),
            "HeapSize": round(mem_info.heap_size / 1024,2),
            "HeapAlloc": round(mem_info.heap_alloc / 1024,2),
            "HeapFree": round(mem_info.heap_free / 1024,2),
        }

        for k,v in self._sample_data[sec].items():
            self.add_point(k, sec, v)

    def to_dict(self, all: bool = True) -> dict:

        if all:
            return self._sample_data
        else:
            start = self.record_range[0]
            end = self.record_range[1]

            data = {}
            for k,v in self._sample_data.items():
                if start <= k <= end:
                    data[k] = v
            return data

    def from_dict(self, data: dict):
        for sec, data_table in data.items():
            for k, v in data_table.items():
                self.add_point(k,int(sec),v)
            self.flush()



import re
from ppadb.device import Device


def device_info(dev: Device) -> dict:
    prop = dev.get_properties()
    info = {}
    info["品牌"] = prop.get("ro.product.brand", "Unknow")  # 品牌
    info["制造商"] = prop.get("ro.product.manufacturer", "Unknow")  # 制造商
    info["型号"] = prop.get("ro.product.model", "Unknow")  # 型号
    info["名称"] = prop.get("ro.product.name", "Unknow")  # 名称
    info["系统版本"] = prop.get("ro.build.version.release", "Unknow")  # 系统版本
    info["SDK版本"] = prop.get("ro.build.version.sdk", "Unknow")  # SDK版本
    info["首选SDK版本"] = prop.get("ro.product.first_api_level", "Unknow")  # 首选SDK版本

    info["CPU平台"] = prop.get("ro.board.platform", "Unknow")

    info["CPU名称"] = __cpu_name(dev)
    info["CPU架构"] = prop.get("ro.product.cpu.abi", "Unknow")  # CPU架构
    info["CPU核心"] = str(dev.cpu_count())
    freq = cpu_freq(dev)[0]
    info["CPU频率"] = f"{freq['min']/1000}MHZ - {freq['max']/1000}MHZ"

    gpu_info = __gpu_info(dev)
    info["GPU型号"] = f"{gpu_info['manufactor']} {gpu_info['name']}"
    info["OpenGL"] = gpu_info["opengl"]

    ram_info = __ram_info(dev)
    info["RAM"] = f"{ram_info['mem_total']/1024/1024:.2f}GB"
    info["SWAP"] = f"{ram_info['swap_total']/1024/1024:.2f}GB"

    info["ROOT"] = str(not dev.shell("su"))

    info["Serial"] = f"{dev.serial}"

    return info


def __ram_info(dev: Device) -> float:
    mem_total_str = dev.shell("cat /proc/meminfo|grep MemTotal")
    swap_total_str = dev.shell("cat /proc/meminfo|grep SwapTotal")

    mem_total = re.search("\d+", mem_total_str)
    swap_total = re.search("\d+", swap_total_str)

    return {"mem_total": int(mem_total.group()), "swap_total": int(swap_total.group())}


def __cpu_name(dev: Device) -> str:
    try:
        text: str = dev.shell("cat /proc/cpuinfo|grep Hardware")
        name = text.split(":")[1].lstrip()
    except:
        name = "Unknow"
    return name


def __gpu_info(dev: Device) -> dict:
    text: str = dev.shell("dumpsys SurfaceFlinger |grep GLES")
    text = text.split(":")[1]
    manufactor, name, opengl = text.split(",")[:3]
    return {
        "manufactor": manufactor.strip(),
        "name": name.strip(),
        "opengl": opengl.strip(),
    }


def cpu_freq(dev: Device) -> list:
    count = dev.cpu_count()
    freq = []
    for index in range(count):
        cmd_root = f"cat /sys/devices/system/cpu/cpu{index}/cpufreq"
        min = dev.shell(f"{cmd_root}/cpuinfo_min_freq")
        cur = dev.shell(f"{cmd_root}/scaling_cur_freq")
        max = dev.shell(f"{cmd_root}/cpuinfo_max_freq")

        freq.append({"min": int(min), "max": int(max), "cur": int(cur)})

    return freq



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'widgets.ui'
##
## Created by: Qt User Interface Compiler version 6.4.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (
    QCoreApplication,
    QDate,
    QDateTime,
    QLocale,
    QMetaObject,
    QObject,
    QPoint,
    QRect,
    QSize,
    QTime,
    QUrl,
    Qt,
)
from PySide6.QtGui import (
    QBrush,
    QColor,
    QConicalGradient,
    QCursor,
    QFont,
    QFontDatabase,
    QGradient,
    QIcon,
    QImage,
    QKeySequence,
    QLinearGradient,
    QPainter,
    QPalette,
    QPixmap,
    QRadialGradient,
    QTransform,
)
from PySide6.QtWidgets import (
    QApplication,
    QCheckBox,
    QComboBox,
    QFrame,
    QGridLayout,
    QGroupBox,
    QLineEdit,
    QPlainTextEdit,
    QProgressBar,
    QPushButton,
    QRadioButton,
    QScrollBar,
    QSizePolicy,
    QSlider,
    QTextEdit,
    QToolButton,
    QVBoxLayout,
    QWidget,
)
import asset_rc


class Ui_Widgets(object):
    def setupUi(self, Widgets):
        if not Widgets.objectName():
            Widgets.setObjectName("Widgets")
        Widgets.resize(1253, 742)
        icon = QIcon()
        icon.addFile(
            ":/icon_w/assets/svg_white/stack.svg", QSize(), QIcon.Normal, QIcon.Off
        )
        icon.addFile(
            ":/icon_b/assets/svg_blue/stack.svg", QSize(), QIcon.Normal, QIcon.On
        )
        Widgets.setWindowIcon(icon)
        Widgets.setStyleSheet(
            "*{\n"
            "    border-radius:8;\n"
            "    color:#6c7c96;\n"
            "}\n"
            "\n"
            "#container {\n"
            "background-color:#2c313c;\n"
            "}\n"
            "\n"
            "#left_menu_frame{\n"
            "background-color:#1b1e23;\n"
            "}\n"
            "\n"
            "#title_bar_frame{\n"
            "background-color:#343b48;\n"
            "}\n"
            "\n"
            "#status_bar_frame{\n"
            "background-color:#343b48;\n"
            "}\n"
            "\n"
            "#content_frame{\n"
            "background-color:#343b48;\n"
            "}\n"
            "\n"
            "#setting_frame {\n"
            "background-color:#3c4454;\n"
            "}\n"
            "\n"
            "#left_column_frame {\n"
            "background-color:#343b48;\n"
            "}\n"
            "\n"
            "/* \u4fa7\u8fb9\u83dc\u5355\u680f\u6837\u5f0f */\n"
            "\n"
            "#LeftMenu {\n"
            "background-color: #005dfd;\n"
            "}\n"
            "\n"
            "#LeftMenu QPushButton {\n"
            "background-color:#1b1e23;\n"
            "height:40;\n"
            "text-align:left;\n"
            "padding-left:15;\n"
            "padding-top:2;\n"
            "padding-bottom:2;\n"
            "color:#6b7884;\n"
            "}\n"
            "\n"
            "#LeftMenu QPushButton:hover{\n"
            "background-color:#21252d;\n"
            "}\n"
            "\n"
            "#LeftMenu QPushButton:pressed{\n"
            "background-color:#2c313c;\n"
            "}\n"
            "\n"
            "#LeftMenu QPushButton:checked{\n"
            "background-color:#2c313c;\n"
            "}\n"
            ""
            "\n"
            "#LeftMenu QPushButton:focus{\n"
            "border:none;\n"
            "}\n"
            "\n"
            "\n"
            "#LeftMenu #bottom{\n"
            "border-top:1 solid #272c36;\n"
            "border-radius: 0;\n"
            "}\n"
            "\n"
            "#LeftMenu #nav_menu {\n"
            "border-top:1 solid #272c36;\n"
            "border-radius: 0\uff1b\n"
            "}\n"
            "\n"
            "#LeftMenu #scrollArea,#scrollAreaWidgetContents_3{\n"
            "background-color:#1b1e23;\n"
            "border:none;\n"
            "}\n"
            "\n"
            "#LeftMenu #btn_toggle:checked{\n"
            "background-color:#21252d;\n"
            "}\n"
            "\n"
            "\n"
            "/* \u6807\u9898\u680f\u6837\u5f0f */\n"
            "\n"
            "#TitleBar #bg{\n"
            "background-color:#343b48;\n"
            "border-radius:8;\n"
            "}\n"
            "\n"
            "#TitleBar #logo {\n"
            "border: 1px solid #3c4454;\n"
            "padding-right:10px;\n"
            "border-top:none;\n"
            "border-bottom:none;\n"
            "border-left:none;\n"
            "margin-right:5px;\n"
            "border-radius:0;\n"
            "}\n"
            "\n"
            "#TitleBar  #logo_title{\n"
            "color:#77b3f1;\n"
            "font-size:20px;\n"
            "font-weight:bold;\n"
            "}\n"
            "\n"
            "#TitleBar  #lb_title{\n"
            "font-size:14px;\n"
            "color:#6c7c96;\n"
            "}\n"
            "\n"
            "#TitleBar  QToolButton{\n"
            "background-color:#343b48;\n"
            "border:none;\n"
            ""
            "border-radius:4;\n"
            "padding: 4;\n"
            "}\n"
            "\n"
            "#TitleBar QToolButton:hover,\n"
            "#TitleBar QToolButton:checked{\n"
            "background-color:#3c4454;\n"
            "}\n"
            "\n"
            "\n"
            "#TitleBar QToolButton:pressed{\n"
            "background-color:#e2e9f7;\n"
            "}\n"
            "\n"
            "#TitleBar #tool{\n"
            "border:1px solid #3c4454;\n"
            "border-top:none;\n"
            "border-bottom:none;\n"
            "margin-right:5px;\n"
            "border-radius:0;\n"
            "}\n"
            "\n"
            "\n"
            "#LeftColumn{\n"
            "background-color:#343b48;\n"
            "}\n"
            "\n"
            "#LeftColumn #top{\n"
            "background-color: #3c4454;\n"
            "}\n"
            "\n"
            "#leftColumn #icon{\n"
            "\n"
            "}\n"
            "\n"
            "#LeftColumn #title{\n"
            "color:#6c7c96;\n"
            "}\n"
            "\n"
            "#LeftColumn QToolButton{\n"
            "border:none;\n"
            "padding:4;\n"
            "background-color: #3c4454;\n"
            "}\n"
            "\n"
            "#LeftColumn #btn_close:hover{\n"
            "background-color:#343b48;\n"
            "}\n"
            "\n"
            "#LeftColumn #btn_close:pressed{\n"
            "background-color:#1b1e23;\n"
            "}\n"
            "\n"
            "\n"
            "/* \u901a\u7528\u63a7\u4ef6 */\n"
            "\n"
            "QToolTip { \n"
            "color:#6b7884;\n"
            "background-color: #1b1e23; \n"
            "border-left: 2px solid #4f9fee;\n"
            "border-radius:"
            " 8px;\n"
            "}\n"
            "\n"
            "QGroupBox{\n"
            "	border: 2px solid gray; \n"
            "    border-radius: 3px; \n"
            "    margin:10;\n"
            "}\n"
            "\n"
            "QGroupBox::title { \n"
            "    subcontrol-position: center top; /* position at the top left*/ \n"
            "	subcontrol-origin: border;\n"
            "	margin-top:-20px;\n"
            "\n"
            "}\n"
            "\n"
            "/* Button */\n"
            "QAbstractButton{\n"
            "background-color:#1b1e23;\n"
            "border:none;\n"
            "border-radius:4;\n"
            "padding: 4;\n"
            "outline:none;\n"
            "width:36;\n"
            "height:36;\n"
            "}\n"
            "\n"
            "QAbstractButton:hover,\n"
            "QAbstractButton:checked{\n"
            "background-color:#21252d;\n"
            "}\n"
            "\n"
            "\n"
            "QAbstractButton:pressed{\n"
            "background-color:#272c36;\n"
            "}\n"
            "\n"
            "QAbstractButton:disabled{\n"
            "background-color: #272c36;\n"
            "}\n"
            "\n"
            "QAbstractButton:checked{\n"
            "background-color:#568af2;\n"
            "color:black;\n"
            "}\n"
            "\n"
            "QAbstractButton:focus{\n"
            "border: 2px solid #568af2;\n"
            "}\n"
            "\n"
            "QAbstractButton[style~='success']:checked{\n"
            "    background-color: #67c23a;\n"
            "}\n"
            "QAbstractButton[style~='warning']:checked{\n"
            "    background-col"
            "or: #e6a23c;\n"
            "}\n"
            "QAbstractButton[style~='danger']:checked{\n"
            "    background-color: #f56c6c;\n"
            "}\n"
            "QAbstractButton[style~='info']:checked{\n"
            "    background-color: #909399;\n"
            "}\n"
            "\n"
            "\n"
            "/* CheckBox */\n"
            "\n"
            "QCheckBox::indicator{\n"
            "    border:2px solid #6c7c96;\n"
            "    border-radius: 2px;\n"
            "    width:10px;\n"
            "    height:10px;\n"
            "    margin-left:8px;\n"
            "}\n"
            "\n"
            "QCheckBox::indicator:checked{\n"
            "    border:2px solid black;\n"
            "    image: url(:/icon/assets/svg/checkmark.svg);\n"
            "}\n"
            "\n"
            "QRadioButton::indicator{\n"
            "    border:1px solid #6c7c96;\n"
            "    background-color:#6c7c96;\n"
            "    border-radius: 4px;\n"
            "    width:10px;\n"
            "    height:10px;\n"
            "    margin-left:8px;\n"
            "}\n"
            "\n"
            "QRadioButton::indicator:checked{\n"
            "    border:1px solid #fff;\n"
            "    background-color:black;\n"
            "}\n"
            "\n"
            "/* TextEdit */\n"
            "QPlainTextEdit,\n"
            "QTextEdit,\n"
            "QLineEdit{\n"
            "    background-color: #1b1e23;\n"
            "    padding:8px;\n"
            "}\n"
            "\n"
            "QPlainTextEdit:focus,\n"
            "QTextEdit:focus,\n"
            "QLine"
            "Edit:focus{\n"
            "    border: 2px solid #568af2;\n"
            "}\n"
            "\n"
            "QPlainTextEdit:read-only,\n"
            "QTextEdit:read-only,\n"
            "QLineEdit:read-only{\n"
            "    border:none;\n"
            "}\n"
            "\n"
            "/* ProgressBar */\n"
            "QProgressBar{\n"
            "    background-color: #1b1e23;\n"
            "    text-align:center;\n"
            "    font-weight: bold;\n"
            "}\n"
            "\n"
            "QProgressBar::chunk:horizontal{\n"
            "    background-color: #4f9fee;\n"
            "    border-radius: 8px;\n"
            "    border-top-right-radius: 0;\n"
            "    border-bottom-right-radius:0;\n"
            "}\n"
            "QProgressBar::chunk:vertical{\n"
            "    background-color: #4f9fee;\n"
            "    border-radius: 8px;\n"
            "    border-top-left-radius: 0;\n"
            "    border-top-right-radius:0;\n"
            "}\n"
            "\n"
            "\n"
            "/* ScrollBar */\n"
            "QScrollBar:horizontal{\n"
            "    background-color: #2c313c;\n"
            "    border-radius: 0;\n"
            "    border:none;\n"
            "    max-height: 8px;\n"
            "}\n"
            "\n"
            "QScrollBar::handle:horizontal{\n"
            "    background-color:#568af2;\n"
            "    border-radius:4px;\n"
            "    min-width: 25px;\n"
            "}\n"
            "\n"
            "QScrollBar::add-page:horizontal{\n"
            "    background-"
            "color:none;\n"
            "}\n"
            "QScrollBar::sub-page:horizontal{\n"
            "    background-color:none;\n"
            "}\n"
            "\n"
            "QScrollBar::add-line:horizontal{\n"
            "    background-color:#272c36;\n"
            "    border:none;\n"
            "    width:0px;\n"
            "    border-top-right-radius: 4px;\n"
            "    border-bottom-right-radius: 4px;\n"
            "    subcontrol-posistion:right;\n"
            "    subcontrol-origin:margin;\n"
            "}\n"
            "QScrollBar::sub-line:horizontal{\n"
            "    background-color:#272c36;\n"
            "    border:none;\n"
            "    width:0px;\n"
            "    border-top-right-radius: 4px;\n"
            "    border-bottom-right-radius: 4px;\n"
            "    subcontrol-posistion:right;\n"
            "    subcontrol-origin:margin;\n"
            "}\n"
            "\n"
            "/* Vertical */\n"
            "QScrollBar:vertical{\n"
            "    background-color: #2c313c;\n"
            "    border-radius: 0;\n"
            "    border:none;\n"
            "    max-width: 8px;\n"
            "}\n"
            "\n"
            "QScrollBar::handle:vertical{\n"
            "    background-color:#568af2;\n"
            "    border-radius:4px;\n"
            "    min-height: 25px;\n"
            "}\n"
            "\n"
            "QScrollBar::add-page:vertical{\n"
            "    background-color:none;\n"
            "}\n"
            "QScrollBar::sub-page:ve"
            "rtical{\n"
            "    background-color:none;\n"
            "}\n"
            "\n"
            "QScrollBar::add-line:vertical{\n"
            "    background-color:#272c36;\n"
            "    border:none;\n"
            "    width:0px;\n"
            "    border-top-right-radius: 4px;\n"
            "    border-top-left-radius: 4px;\n"
            "    subcontrol-posistion:right;\n"
            "    subcontrol-origin:margin;\n"
            "}\n"
            "QScrollBar::sub-line:vertical{\n"
            "    background-color:#272c36;\n"
            "    border:none;\n"
            "    width:0px;\n"
            "    border-top-right-radius: 4px;\n"
            "    border-top-left-radius: 4px;\n"
            "    subcontrol-posistion:right;\n"
            "    subcontrol-origin:margin;\n"
            "}\n"
            "\n"
            "\n"
            "/* Slider */\n"
            "\n"
            "QSlider:horizontal{\n"
            "    margin:8;\n"
            "}\n"
            "\n"
            "QSlider::groove:horizontal{\n"
            "    background-color: #1b1e23;\n"
            "    border-radius: 4px;\n"
            "    margin:0px;\n"
            "    height: 10px;\n"
            "}\n"
            "QSlider::groove:horizontal:hover{\n"
            "    background-color: #21252d;\n"
            "}\n"
            "\n"
            "QSlider::handle:horizontal{\n"
            "    border:none;\n"
            "    height:16px;\n"
            "    width:16px;\n"
            "    margin: -3px;\n"
            "    border-radius: 8px;"
            "\n"
            "    background-color: #4f9fee;\n"
            "}\n"
            "\n"
            "QSlider:vertical{\n"
            "    margin:8;\n"
            "}\n"
            "\n"
            "QSlider::groove:vertical{\n"
            "    background-color: #1b1e23;\n"
            "    border-radius: 4px;\n"
            "    margin:0px;\n"
            "    width: 10px;\n"
            "}\n"
            "QSlider::groove:vertical:hover{\n"
            "    background-color: #21252d;\n"
            "}\n"
            "\n"
            "QSlider::handle:vertical{\n"
            "    border:none;\n"
            "    height:16px;\n"
            "    width:16px;\n"
            "    margin: -3px;\n"
            "    border-radius: 8px;\n"
            "    background-color: #4f9fee;\n"
            "}\n"
            "\n"
            "\n"
            "/* ComboBox */\n"
            "\n"
            "QComboBox{\n"
            "    background-color: #1b1e23;\n"
            "    padding:8px;\n"
            "    selection-background-color:transparent;\n"
            "}\n"
            "\n"
            "QComboBox::drop-down{\n"
            "    border-top-right-radius: 8px;\n"
            "    border-bottom-right-radius: 8px;\n"
            "    subcontrol-origin:padding;\n"
            "    subcontrol-position:center right;\n"
            "    background-color:#1b1e23;\n"
            "    width:10px;\n"
            "    padding:10px;\n"
            "}\n"
            "\n"
            "QComboBox::drop-down:hover{\n"
            "    background-color:black;\n"
            "}\n"
            "\n"
            "QComboBox:"
            ":down-arrow{\n"
            "    image:url(:/icon_w/assets/svg_white/circle-down.svg);\n"
            "    width: 18px;\n"
            "    height:18px;\n"
            "}\n"
            "\n"
            "\n"
            "/* TabWidget */\n"
            "QTabWidget::pane { /* The tab widget frame */\n"
            "    border-top: 2px solid #1b1e23;\n"
            "    background-color:#1b1e23;\n"
            "    border-radius: 8px;\n"
            "    border-top-left-radius:0px;\n"
            "}\n"
            "\n"
            "QTabWidget::tab-bar {\n"
            "    left: 0px; /* move to the right by 5px */\n"
            "}\n"
            "\n"
            "/* Style the tab using the tab sub-control. Note that\n"
            "    it reads QTabBar _not_ QTabWidget */\n"
            "QTabBar::tab {\n"
            "    background: #1b1e23;\n"
            "    border-bottom-color: #1b1e23; \n"
            "    border-top-left-radius: 8px;\n"
            "    border-top-right-radius: 8px;\n"
            "    min-width: 8ex;\n"
            "    padding: 8px;\n"
            "}\n"
            "\n"
            "QTabBar::tab:selected, QTabBar::tab:hover {\n"
            "    background: #568af2;\n"
            "    color:black;\n"
            "}\n"
            "\n"
            "QTabBar::tab:!selected {\n"
            "    margin-top: 2px; /* make non-selected tabs look smaller */\n"
            "}\n"
            ""
        )
        self.verticalLayout = QVBoxLayout(Widgets)
        self.verticalLayout.setObjectName("verticalLayout")
        self.frame = QFrame(Widgets)
        self.frame.setObjectName("frame")
        self.frame.setFrameShape(QFrame.StyledPanel)
        self.frame.setFrameShadow(QFrame.Raised)
        self.horizontalLayoutWidget = QWidget(self.frame)
        self.horizontalLayoutWidget.setObjectName("horizontalLayoutWidget")
        self.horizontalLayoutWidget.setGeometry(QRect(20, 645, 191, 51))
        self.gridLayout_2 = QGridLayout(self.horizontalLayoutWidget)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.gridLayout_2.setContentsMargins(0, 0, 0, 0)
        self.btn_reload = QPushButton(self.horizontalLayoutWidget)
        self.btn_reload.setObjectName("btn_reload")
        sizePolicy = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btn_reload.sizePolicy().hasHeightForWidth())
        self.btn_reload.setSizePolicy(sizePolicy)
        self.btn_reload.setMinimumSize(QSize(40, 0))

        self.gridLayout_2.addWidget(self.btn_reload, 0, 0, 1, 1)

        self.groupBox_2 = QGroupBox(self.frame)
        self.groupBox_2.setObjectName("groupBox_2")
        self.groupBox_2.setGeometry(QRect(230, 10, 220, 291))
        self.groupBox_2.setMinimumSize(QSize(220, 0))
        self.gridLayout = QGridLayout(self.groupBox_2)
        self.gridLayout.setObjectName("gridLayout")
        self.toolButton_4 = QToolButton(self.groupBox_2)
        self.toolButton_4.setObjectName("toolButton_4")
        self.toolButton_4.setMinimumSize(QSize(40, 40))
        icon1 = QIcon()
        icon1.addFile(
            ":/icon_b/assets/svg_blue/IcoMoon.svg", QSize(), QIcon.Normal, QIcon.Off
        )
        icon1.addFile(":/icon/assets/svg/IcoMoon.svg", QSize(), QIcon.Normal, QIcon.On)
        self.toolButton_4.setIcon(icon1)
        self.toolButton_4.setCheckable(True)
        self.toolButton_4.setChecked(True)

        self.gridLayout.addWidget(self.toolButton_4, 0, 1, 1, 1)

        self.toolButton = QToolButton(self.groupBox_2)
        self.toolButton.setObjectName("toolButton")
        self.toolButton.setMinimumSize(QSize(40, 40))
        icon2 = QIcon()
        icon2.addFile(
            ":/icon_w/assets/svg_white/IcoMoon.svg", QSize(), QIcon.Normal, QIcon.Off
        )
        icon2.addFile(":/icon/assets/svg/IcoMoon.svg", QSize(), QIcon.Normal, QIcon.On)
        icon2.addFile(
            ":/icon/assets/svg/IcoMoon.svg", QSize(), QIcon.Disabled, QIcon.Off
        )
        self.toolButton.setIcon(icon2)

        self.gridLayout.addWidget(self.toolButton, 0, 0, 1, 1)

        self.toolButton_3 = QToolButton(self.groupBox_2)
        self.toolButton_3.setObjectName("toolButton_3")
        self.toolButton_3.setMinimumSize(QSize(40, 40))
        self.toolButton_3.setIcon(icon1)
        self.toolButton_3.setCheckable(True)
        self.toolButton_3.setChecked(True)

        self.gridLayout.addWidget(self.toolButton_3, 1, 1, 1, 1)

        self.toolButton_2 = QToolButton(self.groupBox_2)
        self.toolButton_2.setObjectName("toolButton_2")
        self.toolButton_2.setMinimumSize(QSize(40, 40))
        icon3 = QIcon()
        icon3.addFile(
            ":/icon_w/assets/svg_white/IcoMoon.svg", QSize(), QIcon.Normal, QIcon.Off
        )
        icon3.addFile(":/icon/assets/svg/IcoMoon.svg", QSize(), QIcon.Normal, QIcon.On)
        self.toolButton_2.setIcon(icon3)
        self.toolButton_2.setCheckable(True)
        self.toolButton_2.setChecked(True)

        self.gridLayout.addWidget(self.toolButton_2, 2, 0, 1, 1)

        self.toolButton_6 = QToolButton(self.groupBox_2)
        self.toolButton_6.setObjectName("toolButton_6")
        self.toolButton_6.setMinimumSize(QSize(40, 40))
        self.toolButton_6.setIcon(icon3)
        self.toolButton_6.setCheckable(True)

        self.gridLayout.addWidget(self.toolButton_6, 1, 0, 1, 1)

        self.toolButton_5 = QToolButton(self.groupBox_2)
        self.toolButton_5.setObjectName("toolButton_5")
        self.toolButton_5.setMinimumSize(QSize(40, 40))
        icon4 = QIcon()
        icon4.addFile(
            ":/icon_w/assets/svg_white/heart.svg", QSize(), QIcon.Normal, QIcon.Off
        )
        icon4.addFile(":/icon/assets/svg/heart.svg", QSize(), QIcon.Normal, QIcon.On)
        self.toolButton_5.setIcon(icon4)
        self.toolButton_5.setCheckable(True)
        self.toolButton_5.setChecked(True)

        self.gridLayout.addWidget(self.toolButton_5, 2, 1, 1, 1)

        self.toolButton_7 = QToolButton(self.groupBox_2)
        self.toolButton_7.setObjectName("toolButton_7")
        self.toolButton_7.setEnabled(False)
        self.toolButton_7.setMinimumSize(QSize(40, 40))
        icon5 = QIcon()
        icon5.addFile(":/icon/assets/svg/IcoMoon.svg", QSize(), QIcon.Normal, QIcon.Off)
        icon5.addFile(
            ":/icon_w/assets/svg_white/IcoMoon.svg", QSize(), QIcon.Normal, QIcon.On
        )
        self.toolButton_7.setIcon(icon5)

        self.gridLayout.addWidget(self.toolButton_7, 3, 0, 1, 1)

        self.toolButton_8 = QToolButton(self.groupBox_2)
        self.toolButton_8.setObjectName("toolButton_8")
        self.toolButton_8.setMinimumSize(QSize(40, 40))
        self.toolButton_8.setIcon(icon1)
        self.toolButton_8.setCheckable(True)
        self.toolButton_8.setChecked(True)

        self.gridLayout.addWidget(self.toolButton_8, 3, 1, 1, 1)

        self.groupBox_3 = QGroupBox(self.frame)
        self.groupBox_3.setObjectName("groupBox_3")
        self.groupBox_3.setGeometry(QRect(470, 10, 471, 291))
        self.groupBox_3.setMinimumSize(QSize(220, 0))
        self.horizontalScrollBar = QScrollBar(self.groupBox_3)
        self.horizontalScrollBar.setObjectName("horizontalScrollBar")
        self.horizontalScrollBar.setGeometry(QRect(30, 30, 181, 8))
        self.horizontalScrollBar.setOrientation(Qt.Horizontal)
        self.verticalScrollBar = QScrollBar(self.groupBox_3)
        self.verticalScrollBar.setObjectName("verticalScrollBar")
        self.verticalScrollBar.setGeometry(QRect(110, 70, 8, 181))
        self.verticalScrollBar.setOrientation(Qt.Vertical)
        self.horizontalSlider = QSlider(self.groupBox_3)
        self.horizontalSlider.setObjectName("horizontalSlider")
        self.horizontalSlider.setGeometry(QRect(260, 30, 160, 22))
        self.horizontalSlider.setOrientation(Qt.Horizontal)
        self.verticalSlider = QSlider(self.groupBox_3)
        self.verticalSlider.setObjectName("verticalSlider")
        self.verticalSlider.setGeometry(QRect(330, 70, 22, 160))
        self.verticalSlider.setOrientation(Qt.Vertical)
        self.groupBox_4 = QGroupBox(self.frame)
        self.groupBox_4.setObjectName("groupBox_4")
        self.groupBox_4.setGeometry(QRect(10, 310, 220, 128))
        self.groupBox_4.setMinimumSize(QSize(220, 0))
        self.verticalLayout_3 = QVBoxLayout(self.groupBox_4)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.checkBox = QCheckBox(self.groupBox_4)
        self.checkBox.setObjectName("checkBox")
        self.checkBox.setMinimumSize(QSize(40, 40))

        self.verticalLayout_3.addWidget(self.checkBox)

        self.radioButton = QRadioButton(self.groupBox_4)
        self.radioButton.setObjectName("radioButton")
        self.radioButton.setMinimumSize(QSize(40, 40))

        self.verticalLayout_3.addWidget(self.radioButton)

        self.groupBox_5 = QGroupBox(self.frame)
        self.groupBox_5.setObjectName("groupBox_5")
        self.groupBox_5.setGeometry(QRect(230, 310, 491, 321))
        self.lineEdit = QLineEdit(self.groupBox_5)
        self.lineEdit.setObjectName("lineEdit")
        self.lineEdit.setGeometry(QRect(30, 30, 141, 40))
        self.lineEdit.setMinimumSize(QSize(0, 40))
        self.textEdit = QTextEdit(self.groupBox_5)
        self.textEdit.setObjectName("textEdit")
        self.textEdit.setGeometry(QRect(40, 100, 104, 71))
        self.plainTextEdit = QPlainTextEdit(self.groupBox_5)
        self.plainTextEdit.setObjectName("plainTextEdit")
        self.plainTextEdit.setGeometry(QRect(40, 200, 104, 71))
        self.lineEdit_2 = QLineEdit(self.groupBox_5)
        self.lineEdit_2.setObjectName("lineEdit_2")
        self.lineEdit_2.setGeometry(QRect(170, 30, 141, 40))
        self.lineEdit_2.setMinimumSize(QSize(0, 40))
        self.textEdit_2 = QTextEdit(self.groupBox_5)
        self.textEdit_2.setObjectName("textEdit_2")
        self.textEdit_2.setGeometry(QRect(180, 100, 104, 71))
        self.plainTextEdit_2 = QPlainTextEdit(self.groupBox_5)
        self.plainTextEdit_2.setObjectName("plainTextEdit_2")
        self.plainTextEdit_2.setGeometry(QRect(180, 200, 104, 71))
        self.lineEdit_3 = QLineEdit(self.groupBox_5)
        self.lineEdit_3.setObjectName("lineEdit_3")
        self.lineEdit_3.setGeometry(QRect(320, 30, 141, 40))
        self.lineEdit_3.setMinimumSize(QSize(0, 40))
        self.lineEdit_3.setReadOnly(True)
        self.textEdit_3 = QTextEdit(self.groupBox_5)
        self.textEdit_3.setObjectName("textEdit_3")
        self.textEdit_3.setGeometry(QRect(330, 100, 104, 71))
        self.textEdit_3.setReadOnly(True)
        self.plainTextEdit_3 = QPlainTextEdit(self.groupBox_5)
        self.plainTextEdit_3.setObjectName("plainTextEdit_3")
        self.plainTextEdit_3.setGeometry(QRect(330, 200, 104, 71))
        self.plainTextEdit_3.setReadOnly(True)
        self.groupBox_6 = QGroupBox(self.frame)
        self.groupBox_6.setObjectName("groupBox_6")
        self.groupBox_6.setGeometry(QRect(720, 310, 171, 321))
        self.progressBar_2 = QProgressBar(self.groupBox_6)
        self.progressBar_2.setObjectName("progressBar_2")
        self.progressBar_2.setGeometry(QRect(70, 120, 21, 151))
        self.progressBar_2.setValue(50)
        self.progressBar_2.setOrientation(Qt.Vertical)
        self.progressBar = QProgressBar(self.groupBox_6)
        self.progressBar.setObjectName("progressBar")
        self.progressBar.setGeometry(QRect(30, 50, 118, 23))
        self.progressBar.setValue(50)
        self.progressBar.setOrientation(Qt.Horizontal)
        self.groupBox = QGroupBox(self.frame)
        self.groupBox.setObjectName("groupBox")
        self.groupBox.setGeometry(QRect(11, 11, 220, 291))
        self.groupBox.setFlat(False)
        self.groupBox.setCheckable(False)
        self.verticalLayout_2 = QVBoxLayout(self.groupBox)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.pushButton_2 = QPushButton(self.groupBox)
        self.pushButton_2.setObjectName("pushButton_2")
        self.pushButton_2.setMinimumSize(QSize(40, 40))
        self.pushButton_2.setMaximumSize(QSize(16777215, 16777215))

        self.verticalLayout_2.addWidget(self.pushButton_2)

        self.pushButton = QPushButton(self.groupBox)
        self.pushButton.setObjectName("pushButton")
        self.pushButton.setMinimumSize(QSize(40, 40))
        self.pushButton.setMaximumSize(QSize(16777215, 16777215))
        icon6 = QIcon()
        icon6.addFile(
            ":/icon_w/assets/svg_white/IE.svg", QSize(), QIcon.Normal, QIcon.Off
        )
        icon6.addFile(":/icon/assets/svg/IE.svg", QSize(), QIcon.Normal, QIcon.On)
        self.pushButton.setIcon(icon6)

        self.verticalLayout_2.addWidget(self.pushButton)

        self.pushButton_3 = QPushButton(self.groupBox)
        self.pushButton_3.setObjectName("pushButton_3")
        self.pushButton_3.setMinimumSize(QSize(40, 40))
        self.pushButton_3.setMaximumSize(QSize(16777215, 16777215))
        self.pushButton_3.setIcon(icon6)
        self.pushButton_3.setCheckable(True)

        self.verticalLayout_2.addWidget(self.pushButton_3)

        self.pushButton_4 = QPushButton(self.groupBox)
        self.pushButton_4.setObjectName("pushButton_4")
        self.pushButton_4.setMinimumSize(QSize(40, 40))
        self.pushButton_4.setMaximumSize(QSize(16777215, 16777215))
        self.pushButton_4.setIcon(icon6)
        self.pushButton_4.setCheckable(True)
        self.pushButton_4.setChecked(True)

        self.verticalLayout_2.addWidget(self.pushButton_4)

        self.pushButton_5 = QPushButton(self.groupBox)
        self.pushButton_5.setObjectName("pushButton_5")
        self.pushButton_5.setEnabled(False)
        self.pushButton_5.setMinimumSize(QSize(40, 40))
        self.pushButton_5.setMaximumSize(QSize(16777215, 16777215))
        self.pushButton_5.setIcon(icon6)

        self.verticalLayout_2.addWidget(self.pushButton_5)

        self.comboBox = QComboBox(self.frame)
        self.comboBox.setObjectName("comboBox")
        self.comboBox.setGeometry(QRect(920, 330, 131, 36))
        self.comboBox.setMinimumSize(QSize(0, 36))
        self.comboBox.setEditable(False)
        self.comboBox.setModelColumn(0)
        self.btn_notify = QPushButton(self.frame)
        self.btn_notify.setObjectName("btn_notify")
        self.btn_notify.setGeometry(QRect(30, 440, 181, 41))
        self.btn_notify_2 = QPushButton(self.frame)
        self.btn_notify_2.setObjectName("btn_notify_2")
        self.btn_notify_2.setGeometry(QRect(30, 490, 181, 41))

        self.verticalLayout.addWidget(self.frame)

        self.retranslateUi(Widgets)

        QMetaObject.connectSlotsByName(Widgets)

    # setupUi

    def retranslateUi(self, Widgets):
        Widgets.setWindowTitle(
            QCoreApplication.translate("Widgets", "\u63a7\u4ef6\u53c2\u8003", None)
        )
        self.btn_reload.setText(
            QCoreApplication.translate("Widgets", "\u5237\u65b0\u6837\u5f0f", None)
        )
        self.groupBox_2.setTitle(
            QCoreApplication.translate("Widgets", "ToolButton", None)
        )
        self.toolButton_4.setText(QCoreApplication.translate("Widgets", "...", None))
        self.toolButton_4.setProperty(
            "style", QCoreApplication.translate("Widgets", "success", None)
        )
        self.toolButton.setText(QCoreApplication.translate("Widgets", "...", None))
        self.toolButton_3.setText(QCoreApplication.translate("Widgets", "...", None))
        self.toolButton_3.setProperty(
            "style", QCoreApplication.translate("Widgets", "warning", None)
        )
        self.toolButton_2.setText(QCoreApplication.translate("Widgets", "...", None))
        self.toolButton_6.setText(QCoreApplication.translate("Widgets", "...", None))
        self.toolButton_5.setText(QCoreApplication.translate("Widgets", "...", None))
        self.toolButton_5.setProperty(
            "style", QCoreApplication.translate("Widgets", "danger", None)
        )
        self.toolButton_7.setText(QCoreApplication.translate("Widgets", "...", None))
        self.toolButton_8.setText(QCoreApplication.translate("Widgets", "...", None))
        self.toolButton_8.setProperty(
            "style", QCoreApplication.translate("Widgets", "info", None)
        )
        self.groupBox_3.setTitle(QCoreApplication.translate("Widgets", "Bar", None))
        self.groupBox_4.setTitle(
            QCoreApplication.translate("Widgets", "CheckBox", None)
        )
        self.checkBox.setText(QCoreApplication.translate("Widgets", "CheckBox", None))
        self.radioButton.setText(
            QCoreApplication.translate("Widgets", "RadioButton", None)
        )
        self.groupBox_5.setTitle(
            QCoreApplication.translate("Widgets", "TextEdit", None)
        )
        self.lineEdit.setText(
            QCoreApplication.translate("Widgets", "hello world", None)
        )
        self.textEdit.setHtml(
            QCoreApplication.translate(
                "Widgets",
                '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0//EN" "http://www.w3.org/TR/REC-html40/strict.dtd">\n'
                '<html><head><meta name="qrichtext" content="1" /><meta charset="utf-8" /><style type="text/css">\n'
                "p, li { white-space: pre-wrap; }\n"
                "hr { height: 1px; border-width: 0; }\n"
                "</style></head><body style=\" font-family:'Microsoft YaHei UI'; font-size:9pt; font-weight:400; font-style:normal;\">\n"
                '<p style=" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;">hello<span style=" font-size:14pt;"> world</span></p></body></html>',
                None,
            )
        )
        self.plainTextEdit.setPlainText(
            QCoreApplication.translate("Widgets", "hello world", None)
        )
        self.lineEdit_2.setPlaceholderText(
            QCoreApplication.translate("Widgets", "place holder", None)
        )
        self.textEdit_2.setPlaceholderText(
            QCoreApplication.translate("Widgets", "place holder", None)
        )
        self.plainTextEdit_2.setPlaceholderText(
            QCoreApplication.translate("Widgets", "placeholder", None)
        )
        self.lineEdit_3.setText(
            QCoreApplication.translate("Widgets", "hello world", None)
        )
        self.textEdit_3.setHtml(
            QCoreApplication.translate(
                "Widgets",
                '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0//EN" "http://www.w3.org/TR/REC-html40/strict.dtd">\n'
                '<html><head><meta name="qrichtext" content="1" /><meta charset="utf-8" /><style type="text/css">\n'
                "p, li { white-space: pre-wrap; }\n"
                "hr { height: 1px; border-width: 0; }\n"
                "</style></head><body style=\" font-family:'Microsoft YaHei UI'; font-size:9pt; font-weight:400; font-style:normal;\">\n"
                '<p style=" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;">hello<span style=" font-size:14pt;"> world</span></p></body></html>',
                None,
            )
        )
        self.plainTextEdit_3.setPlainText(
            QCoreApplication.translate("Widgets", "hello world", None)
        )
        self.groupBox_6.setTitle(
            QCoreApplication.translate("Widgets", "ProgressBar", None)
        )
        self.groupBox.setTitle(
            QCoreApplication.translate("Widgets", "PushButton", None)
        )
        self.pushButton_2.setText(
            QCoreApplication.translate("Widgets", "PushButton", None)
        )
        self.pushButton.setText(
            QCoreApplication.translate("Widgets", "PushButton", None)
        )
        self.pushButton_3.setText(
            QCoreApplication.translate("Widgets", "PushButton", None)
        )
        self.pushButton_4.setText(
            QCoreApplication.translate("Widgets", "PushButton", None)
        )
        self.pushButton_5.setText(
            QCoreApplication.translate("Widgets", "PushButton", None)
        )
        self.comboBox.setCurrentText("")
        self.btn_notify.setText(
            QCoreApplication.translate(
                "Widgets", "\u6d88\u606f\u63d0\u793a\u6837\u5f0f1", None
            )
        )
        self.btn_notify_2.setText(
            QCoreApplication.translate(
                "Widgets", "\u6d88\u606f\u63d0\u793a\u6837\u5f0f2", None
            )
        )
        self.btn_notify_2.setProperty(
            "style", QCoreApplication.translate("Widgets", "success", None)
        )

    # retranslateUi



from PySide6.QtWidgets import QWidget, QGraphicsDropShadowEffect
from PySide6.QtCore import Slot, QPropertyAnimation, QEasingCurve
from .ui_left_column import Ui_LeftColumn


class LeftColumn(QWidget, Ui_LeftColumn):
    def __init__(self, parent) -> None:
        super().__init__(parent)
        self.setupUi(self)
        self.setStyleSheet("")



import enum

class ButtonStyle(enum.Enum):
    primary = "primary"
    success = "success"
    warning = "warning"
    danger = "danger"
    info = "info"

Color = {   
    'primary' : '#1b1e23',
    'success' : '#67c23a',
    'warning' : '#e6a23c',
    'danger'  : '#f56c6c',
    'info'    : '#909399'
}


import logging
from ppadb.device import Device


log = logging.getLogger(__name__)


class FpsSampler:
    SURFACE_PARTTERN = "^{package_name}/com.[\w\.]+#\d$"

    def __init__(self, device: Device, package_name: str) -> None:
        self.device = device
        self._package_name = package_name
        self._surface_name = None

    @property
    def surface_name(self) -> str:
        result: str = self.device.shell(
            f'dumpsys SurfaceFlinger --list|grep "{self._package_name}"'
        )

        results = result.split("\n")

        # 由上到下扫描出最后一个SurfaceView
        top_surface_view = None
        surface_views = filter(lambda txt: txt.startswith("SurfaceView"), results)
        surface_views = list(surface_views)
        if surface_views:
            top_surface_view = surface_views[-1]

        # 找出包含app_name的SurfaceView
        top_app_surface_view = None
        app_surface_views = filter(
            lambda txt: self._package_name in txt and txt.startswith("SurfaceView"),
            results,
        )
        app_surface_views = list(app_surface_views)
        if app_surface_views:
            top_app_surface_view = app_surface_views[-1]

        target_surface_view = top_app_surface_view or top_surface_view or ""

        return target_surface_view.strip()

    @property
    def data(self) -> dict:
        """
        参考：https://zhuanlan.zhihu.com/p/67056913
        Returns:
            float: 帧率
        """
        data = {"fps": -1, "jank": -1, "big_jank": -1, "*frametimes": [0]}

        if not self.surface_name:
            log.warning("没有找到surface")
            return data

        log.debug(f'dumpsys SurfaceFlinger --latency "{self.surface_name}"')

        result: str = self.device.shell(
            f'dumpsys SurfaceFlinger --latency "{self.surface_name}"'
        )
        refresh_period, data_table = self._parse_data(result)
        if not data_table:
            log.warning(f"{self.surface_name} 没有数据")
            return data

        fps = self._calc_fps(data_table, refresh_period)
        jank, big_jank, frametime = self._calc_jank(data_table, refresh_period)

        return {
            "fps": fps,
            "jank": jank,
            "big_jank": big_jank,
            "*frametimes": frametime,
        }

    def _parse_data(self, result: str):
        lines = result.strip().split("\n")
        refresh_period = float(lines[0])

        data = []
        if len(lines) > 1:
            for line in lines[1:]:
                line_data = [float(v) for v in line.split("\t")]
                data.append(line_data)
        data = list(filter(lambda v: sum(v) > 0, data))

        return refresh_period, data

    def _calc_jank(self, data_table, refresh_period):
        jank_count = 0
        big_jank_count = 0

        # 计算帧间隔，转为ms
        frametimes = [
            round((data_table[i + 1][0] - data_table[i][0]) / pow(10, 6), 2)
            for i in range(len(data_table) - 1)
        ]

        for i in range(3, len(frametimes)):
            pre_three_avg = (
                frametimes[i - 1] + frametimes[i - 2] + frametimes[i - 3]
            ) / 3
            cur_frametime = frametimes[i]
            if cur_frametime > 2 * pre_three_avg:
                if cur_frametime > 83.33:
                    jank_count += 1

                if cur_frametime > 125:
                    big_jank_count += 1

        return jank_count, big_jank_count, frametimes

    def _calc_fps(self, data_table, refresh_period):
        if refresh_period < 0:
            return -1.0

        frame_count = len(data_table)

        if frame_count == 1:
            return 0

        start_time = data_table[0][0]
        end_time = data_table[-1][0]

        duration = end_time - start_time
        # pow(10,9)是 1000000000 ，用来把纳秒转秒

        return frame_count * pow(10, 9) / duration



import os

# todo: 之后要引入dotenv 根据开发和生产环境切换导入这个控件示例页面
from .widgets import Widgets
from .home import Home
from .profiler import Profiler

MODE = os.environ.get("MODE", "development")
print("==========", MODE)

register = [Home, Profiler]
if MODE == "development":
    register.append(Widgets)



import logging
from perfcat.ui.constant import ButtonStyle
from perfcat.ui.page import Page
from perfcat.ui.widgets import notification
from .ui_widgets import Ui_Widgets
from PySide6.QtCore import Qt
from perfcat.ui.widgets.notification import Notification

log = logging.getLogger(__name__)


class Widgets(Page, Ui_Widgets):
    def __init__(self, parent) -> None:
        super().__init__(parent)

        self.setupUi(self)
        self.clear_stylesheet()

        self.comboBox.addItem("x10")

        self.btn_reload.clicked.connect(self.reload_stylesheet)

        self.btn_notify.clicked.connect(self.notify)
        self.btn_notify_2.clicked.connect(self.notify_2)

    def reload_stylesheet(self):
        from perfcat.app import PerfcatApplication

        PerfcatApplication.instance.load_stylesheet()
        self.repaint()
        self.adjustSize()

    def notify_2(self):
        log.debug("弹出提示！")
        Notification(self, msg="这是样式2", style=ButtonStyle.success).show()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :   Chart.py
@Time    :   2022/05/07 16:18:16
@Author  :   Calros Teng 
@Version :   1.0
@Contact :   303359166@qq.com
@License :   (C)Copyright 2017-2018, Xin Yuan Studio
@Desc    :   

[1] 画出每个系列的最大值水位线
"""

# here put the import lib


import logging
import math
import PySide6

from typing import Callable, Union, Dict
from ppadb.device import Device
from PySide6.QtCharts import (
    QChart,
    QChartView,
    QDateTimeAxis,
    QLegendMarker,
    QLineSeries,
    QValueAxis,
    QAbstractSeries,
)
from PySide6.QtCore import (
    QDateTime,
    QPoint,
    QPointF,
    QRect,
    QRectF,
    Qt,
    Signal,
    SignalInstance,
)
from PySide6.QtGui import QBrush, QColor, QMouseEvent, QPainter, QPen, QWheelEvent
from PySide6.QtWidgets import QScrollBar


log = logging.getLogger(__name__)


class MonitorChart(QChartView):
    series_changed: SignalInstance = Signal()
    x_offset_changed: SignalInstance = Signal(int)
    x_max_offset_changed: SignalInstance = Signal(int)
    axis_range_size_changed: SignalInstance = Signal(int)
    mark_line_changed: SignalInstance = Signal(QPoint)

    INIT_RANGE_SIZE = 30

    def __init__(
        self,
        parent,
        y_axis_name="%",
    ):
        super().__init__(parent)
        self.setToolTip("")

        self.setMaximumHeight(300)
        self.setMinimumHeight(240)
        self.setStyleSheet("background-color:transparent;")

        _chart: QChart = QChart()
        _chart.layout().setContentsMargins(0, 0, 4, 0)
        _chart.legend().setMinimumWidth(150)
        _chart.legend().setMaximumWidth(150)

        self._device = None
        self._package_name = None

        self._series_map: Dict[str, QLineSeries] = {}
        self._point_buffer = {}  # 记录点缓冲池，add_point的时候会缓存到这里，然后通过flush函数一起写入serial
        self._value_formatter: Dict[str, Callable[[float], str]] = {}  # 值格式化
        self._mark_line: QPoint = QPoint()  # 标线坐标

        self.tick_count = 0

        self.total_x = 0  # x轴序列最大值
        self._axis_range_size = self.INIT_RANGE_SIZE  # 区间大小
        self._x_offset = 0  # 区间整体偏移

        self.recording = False
        self.record_range = [-1, -1]

        _chart.setTheme(QChart.ChartThemeDark)
        _chart.legend().setAlignment(Qt.AlignRight)

        self.setRenderHint(QPainter.Antialiasing)

        # 时间轴坐标用一个比较笨的方法去实现
        # 开始时间直接选择0时间戳时间，也就是1970年8:00:00
        # 之后采集到的数据以这个时间为基准+秒
        self.axis_x = QDateTimeAxis(self)
        self.update_range()
        self.axis_x.setFormat("mm:ss")
        _chart.addAxis(self.axis_x, Qt.AlignBottom)

        self.axis_y = QValueAxis(self)
        self.axis_y.setRange(0, 100)
        self.axis_y.setTitleText(y_axis_name)
        _chart.addAxis(self.axis_y, Qt.AlignLeft)

        self.axis_x.rangeChanged.connect(self.update_x_offset)

        self.setChart(_chart)

        for mk in _chart.legend().markers():
            mk.clicked.connect(self._on_marker_clicked)

    def create_series(
        self,
        name: str,
        series: QAbstractSeries,
        v_format: Callable[[float], str] = lambda v: v,
    ):
        series.setName(name)
        self.chart().addSeries(series)
        self.chart().setAxisX(self.axis_x, series)
        self.chart().setAxisY(self.axis_y, series)
        self._value_formatter[name] = v_format
        self._series_map[name] = series

    def reset_series_data(self):
        """
        清空系列数据
        """
        for s in self._series_map.values():
            s.clear()

        self.total_x = 0
        self.record_range = [-1, -1]

    def sample(self, sec: int, device: Device, package_name: str):
        """每一tick更新数据，自己实现，然后通过addpoint添加数据点"""
        raise NotImplementedError

    def flush(self):
        """
        刷入采样数据

        _extended_summary_
        """
        for s_name, p in self._point_buffer.items():
            self._series_map[s_name].append(*p)
        self._point_buffer.clear()
        self.update()
        self.series_changed.emit()

    def _base_time(self) -> QDateTime:
        return QDateTime.fromMSecsSinceEpoch(0)

    def update_x_offset(self, min_d: QDateTime, max_d: QDateTime):
        offset = min_d.toSecsSinceEpoch() - self._base_time().toSecsSinceEpoch()
        self._x_offset = max(offset, 0)
        self._axis_range_size = max_d.toSecsSinceEpoch() - min_d.toSecsSinceEpoch()
        self.update_range()

    def axis_x_min(self):
        return self._base_time().addSecs(self._x_offset)

    def axis_x_max(self):
        return self.axis_x_min().addSecs(self._axis_range_size)

    def x_max_offset(self):
        return max(0, self.total_x - self._axis_range_size)

    @property
    def axis_range_size(self):
        """
        这个值是区间范围，每次缩放的时候会改变
        """
        return self._axis_range_size

    @axis_range_size.setter
    def axis_range_size(self, value):
        self._axis_range_size = max(self.INIT_RANGE_SIZE, value)
        self.update_range()

    @property
    def x_offset(self):
        return self._x_offset

    @x_offset.setter
    def x_offset(self, value):
        """
        通过修改这个值平移
        """
        self._x_offset = value
        self.update_range()
        self.x_offset_changed.emit(value)

    def set_x_offset(self, value):
        self.x_offset = value

    def update_range(self):
        self.axis_x.setRange(self.axis_x_min(), self.axis_x_max())

    @property
    def mark_line(self) -> QPoint:
        """
        获取标线坐标

        _extended_summary_

        Returns:
            QPoint: _description_
        """
        return self._mark_line

    @mark_line.setter
    def mark_line(self, point: QPoint):
        # refer to https://stackoverflow.com/a/44078533/9758790
        scene_position = self.mapToScene(point)
        chart_position = self.chart().mapFromScene(scene_position)
        value_at_position = self.chart().mapToValue(chart_position)

        if (
            self.axis_x.min().toMSecsSinceEpoch()
            < value_at_position.x()
            < self.axis_x.max().toMSecsSinceEpoch()
        ):
            self._mark_line = scene_position.toPoint()

        self.update()

    def _on_marker_clicked(self):
        mk: QLegendMarker = self.sender()
        mk.series().setVisible(not mk.series().isVisible())
        mk.setVisible(True)
        log.debug(f"设置图例显隐：{mk.series().name()} {mk.series().isVisible()}")

        # 隐藏系列的时候对图例样式做个表现
        # 参考自官方：https://doc.qt.io/qt-5/qtcharts-legendmarkers-example.html

        alpha = 1.0
        if not mk.series().isVisible():
            alpha = 0.5

        brush = mk.labelBrush()
        color = brush.color()
        color.setAlphaF(alpha)
        brush.setColor(color)
        mk.setLabelBrush(brush)

        brush = mk.brush()
        color = brush.color()
        color.setAlphaF(alpha)
        brush.setColor(color)
        mk.setBrush(brush)

        brush = mk.pen()
        color = brush.color()
        color.setAlphaF(alpha)
        brush.setColor(color)
        mk.setPen(brush)

    def add_point(self, s_name: str, x: float, y: float):
        """
        加数据点
        """

        # series: QLineSeries = self.series_map.get(s_name)
        time = self._base_time().addSecs(x)
        # series.append(time.toMSecsSinceEpoch(), y)

        pre_total_x = self.total_x
        self.total_x = max(self.total_x, x)
        if pre_total_x != self.total_x:
            self.x_max_offset_changed.emit(self.x_max_offset())

        y_max = max(self.axis_y.max(), y)
        self.axis_y.setMax(y_max)

        self._point_buffer[s_name] = (time.toMSecsSinceEpoch(), y)

        self.tick_count = x

        if self.recording:
            self.record_range[1] = self.tick_count

    def drawForeground(
        self,
        painter: QPainter,
        rect: Union[QRectF, QRect],
    ) -> None:
        painter.save()

        # 绘制标线
        _start = self._base_time().addSecs(self.record_range[0])
        _end = self._base_time().addSecs(self.record_range[1])

        _start: QDateTime = max(self.axis_x_min(), min(_start, self.axis_x_max()))
        _end: QDateTime = max(self.axis_x_min(), min(_end, self.axis_x_max()))

        start = (
            self.chart().mapToPosition(QPointF(_start.toMSecsSinceEpoch(), 0)).toPoint()
        )
        end = self.chart().mapToPosition(QPointF(_end.toMSecsSinceEpoch(), 0)).toPoint()
        color = QColor("#007acc")
        color.setAlphaF(0.8)
        pen = QPen(color)
        pen.setWidth(1)
        painter.setPen(pen)
        painter.setBrush(QBrush(color, Qt.SolidPattern))
        painter.drawRect(start.x(), start.y() + 1, end.x() - start.x(), 3)

        if self.mark_line.x() == 0:
            painter.restore()
            return

        # 绘制标线
        pen = QPen(QColor("white"))
        pen.setWidth(1)
        painter.setPen(pen)

        area_rect = self.chart().plotArea()

        p1 = QPointF(self.mark_line.x(), area_rect.top())
        p2 = QPointF(self.mark_line.x(), area_rect.bottom())

        painter.drawLine(p1, p2)

        chart_position = self.chart().mapFromScene(self.mark_line)
        value_at_position = self.chart().mapToValue(chart_position)

        points = {}

        # 绘制值点
        for name, series in self._series_map.items():
            pen2 = QPen(series.color())
            pen2.setWidth(10)
            painter.setPen(pen2)

            # 找到最近的点

            # 遍历所有点

            nearest_left = None
            nearest_right = None
            exact_point = None
            last_diff = 0  # 上次的差值

            for p in series.pointsVector():
                if nearest_left is None:
                    # 如果最左为空，那么先设置最左为p
                    nearest_left = p
                    last_diff = p.x() - value_at_position.x()
                    continue

                if p.x() == value_at_position.x():
                    # 鼠标所在横坐标正好有一个点，就直接退出了
                    exact_point = p
                    nearest_left = None
                    nearest_right = None
                    break

                # 不然就逼近
                diff = p.x() - value_at_position.x()
                if math.copysign(1, diff) + math.copysign(1, last_diff) == 0:
                    nearest_right = p
                    break
                else:
                    nearest_left = p
                    last_diff = diff

            if exact_point:
                painter.drawPoint(
                    self.chart().mapToScene(self.chart().mapToPosition(exact_point))
                )
                points[series.name()] = exact_point
            elif nearest_left and nearest_right:
                # 斜率
                k = (nearest_right.y() - nearest_left.y()) / (
                    nearest_right.x() - nearest_left.x()
                )
                # 插值点
                point_interp_y = nearest_left.y() + k * (
                    value_at_position.x() - nearest_left.x()
                )
                # print(point_interp_y)
                point_interp_x = value_at_position.x()
                point_intrep = QPoint(point_interp_x, point_interp_y)

                painter.drawPoint(self.chart().mapToPosition(point_intrep))
                points[series.name()] = point_intrep

        # 绘制信息label
        painter.setPen(QPen(QColor("white")))
        scene_rect = self.sceneRect()
        text_rect = QRectF(
            p1.x() + 10, p1.y() + 10, scene_rect.width(), scene_rect.height()
        )

        lines = []
        time = self._base_time().addMSecs(int(value_at_position.x()))
        lines.append(time.toString("mm:ss"))
        for name, point in points.items():
            if not self._series_map[name].isVisible():
                continue
            format = self._value_formatter.get(name, lambda v: v)
            lines.append(f"{name}: {format(point.y())}")
        text = "\n".join(lines)

        painter.drawText(text_rect, Qt.AlignLeft, text)

        painter.restore()
        return super().drawForeground(painter, rect)

    def mouseMoveEvent(self, event: QMouseEvent) -> None:
        self.mark_line = event.pos()
        self.mark_line_changed.emit(event.pos())
        return super().mouseMoveEvent(event)

    def wheelEvent(self, event: QWheelEvent) -> None:
        if self.chart().plotArea().contains(event.position()):
            sign = int(math.copysign(1, event.angleDelta().y()))
            self.axis_range_size += sign * 10
            self.axis_range_size_changed.emit(self.axis_range_size)
            self.x_max_offset_changed.emit(self.x_max_offset())
        else:
            # 莫名其妙的会上下滚动，找不到原因，暂时屏蔽掉基类的滚动
            # return super().wheelEvent(event)
            event.ignore()  #  event传进来的时候默认是setaccept(True)的，因此我们要ignore掉让其传递到父widget去处理，比如滚动

    def to_dict(self, all: bool = True) -> dict:
        """
        series 转字典

        _extended_summary_

        Returns:
            dict: _description_
        """
        raise NotImplementedError

    def from_dict(self, data: dict):
        """
        把字典的值读取转series

        _extended_summary_

        Args:
            value (_type_): _description_
        """
        raise NotImplementedError

    def record_enable(self, enable: bool, tick_count=-1):
        self.recording = enable
        if enable:
            self.record_range = [tick_count, tick_count]


if __name__ == "__main__":
    import random
    import sys

    from PySide6.QtCharts import QChart, QChartView, QLineSeries, QValueAxis
    from PySide6.QtCore import Qt, QThread, QTimer, Signal, SignalInstance
    from PySide6.QtGui import QPainter
    from PySide6.QtWidgets import (
        QApplication,
        QHBoxLayout,
        QMainWindow,
        QPushButton,
        QSlider,
        QScrollBar,
        QVBoxLayout,
        QWidget,
    )

    class Win(QWidget):
        def __init__(self, parent=None) -> None:
            super().__init__(parent)
            # self.setStyleSheet("background-color:red")

            self.chart_view1 = MonitorChart(parent, ["CPU"])
            self.chart_view1.setObjectName("c1")
            self.chart_view2 = MonitorChart(parent, ["GPU"])
            self.chart_view2.setObjectName("c2")
            self.chart_views = [self.chart_view1, self.chart_view2]
            self.layout = QVBoxLayout(self)
            self.layout.setContentsMargins(0, 0, 0, 0)
            self.layout.addWidget(self.chart_view1)
            self.layout.addWidget(self.chart_view2)

            for i in range(1000):
                self.chart_view1.add_point("CPU", i, random.randrange(150))
            for i in range(1000):
                self.chart_view2.add_point("GPU", i, random.randrange(150))

            self.h_scrollbar = QScrollBar(Qt.Horizontal, self)
            self.h_scrollbar.setMaximum(self.chart_view1.x_max_offset())
            self.layout.addWidget(self.h_scrollbar)

            self.v = 0

            self.h_scrollbar.valueChanged.connect(self.scroll_changed)
            for c in self.chart_views:
                c.axis_range_size_changed.connect(self.range_size_changed)
                c.mark_line_changed.connect(self.on_mark_line_changed)
            self.startTimer(1000)

        def timerEvent(self, event: PySide6.QtCore.QTimerEvent) -> None:
            return super().timerEvent(event)

        def on_mark_line_changed(self, point):
            sender = self.sender()
            for chartview in self.chart_views:
                if chartview == sender:
                    continue
                chartview.mark_line = sender.mark_line
                print(chartview.objectName(), chartview.mark_line)

        def range_size_changed(self, size):
            sender = self.sender()
            for chartview in self.chart_views:
                if chartview == sender:
                    continue
                chartview.axis_range_size = size

        def scroll_changed(self, v):
            self.chart_view1.set_x_offset = v
            self.chart_view2.set_x_offset = v

    app = QApplication(sys.argv)

    widget = Win()

    widget.resize(800, 400)
    widget.show()
    sys.exit(app.exec())



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :   __init__.py
@Time    :   2022/04/29 12:12:58
@Author  :   Calros Teng 
@Version :   1.0
@Contact :   303359166@qq.com
@License :   (C)Copyright 2017-2018, Xin Yuan Studio
@Desc    :   

linux系统下的 热拔插监控实现

用pyudev的observer来实现
"""

# here put the import lib

import pyudev

from PySide6.QtCore import QCoreApplication
from pyudev.monitor import MonitorObserver
from .base import BaseHotPlugNativeEventFilter


class LinuxHotPlugNatvieEventFilter(BaseHotPlugNativeEventFilter):
    def _action_handler(self, action, device):
        if action == 'add':
            self.device_added.emit()
            self.device_changed.emit()        
        elif action == 'remove':
            self.device_removed.emit()
            self.device_changed.emit()        
            

    def install(self, app: QCoreApplication):
        self.context = pyudev.Context()
        self.observer = MonitorObserver(self.context, self._action_handler)
        self.observer.start()
        return super().install(app)
    
    def uninstall(self, app: QCoreApplication):
        self.observer.stop()
        return super().uninstall(app)


import sys
import logging
from typing import Optional
from ppadb.device import Device

from PySide6.QtWidgets import QApplication, QWidget, QFileDialog, QMessageBox
from PySide6.QtCore import Qt, QSortFilterProxyModel
from .ui_logcat import Ui_Logcat
from .log_model import LogModel, LogSortFilterProxyModel

log = logging.getLogger(__name__)


class LogcatWidget(QWidget, Ui_Logcat):
    def __init__(self, parent: Optional[QWidget] = None) -> None:
        super().__init__(parent)
        self.setupUi(self)

        self._device: Device = None

        self.btn_start.setEnabled(False)

        self.origin_model = LogModel(self)
        self.origin_model.rowsInserted.connect(self._on_scroll_to_bottom)

        self.proxy_model = LogSortFilterProxyModel(self)
        self.proxy_model.setSourceModel(self.origin_model)
        self.proxy_model.setFilterCaseSensitivity(Qt.CaseInsensitive)
        # 过滤内容列
        self.proxy_model.setFilterKeyColumn(-1)

        self.tbv_logs.setModel(self.proxy_model)
        self.btn_start.toggled.connect(self.on_start)

        self._auto_scroll_to_bottom = True
        vsbar = self.tbv_logs.verticalScrollBar()
        vsbar.valueChanged.connect(self._on_vsbar_value_changed)

        # 固定字符匹配
        self.le_search.editingFinished.connect(
            lambda: self.proxy_model.set_filter_by_column(
                self.le_search.text(), LogModel.COL_CONTENT
            )
        )
        self.le_tag.editingFinished.connect(
            lambda: self.proxy_model.set_filter_by_column(
                self.le_tag.text(), LogModel.COL_TAG
            )
        )
        self.cbx_priority.activated.connect(
            lambda: self.proxy_model.set_filter_by_column(
                self.cbx_priority.currentText(), LogModel.COL_LEVEL
            )
        )

        # 列表右键菜单
        self.tbv_logs.addAction(self.action_copy)
        self.action_copy.triggered.connect(self._on_log_copy)

        # 保存内容
        self.btn_save.clicked.connect(self._on_btn_save)

        # 清空内容
        self.btn_clear.clicked.connect(self.origin_model.clear)

        # 初始化前5列的列宽
        self.tbv_logs.setColumnWidth(0, 50)
        self.tbv_logs.setColumnWidth(1, 90)
        self.tbv_logs.setColumnWidth(2, 45)
        self.tbv_logs.setColumnWidth(3, 45)
        self.tbv_logs.setColumnWidth(4, 45)

    @property
    def device(self):
        return self._device

    @device.setter
    def device(self, value: Device):
        self._device = value
        if value == None:
            self.btn_start.setEnabled(False)
        else:
            self.btn_start.setEnabled(True)

    def _on_tag_text_changed(self, text: str):
        self.proxy_model.setFilterKeyColumn(self.origin_model.COL_TAG)
        self.proxy_model.setFilterFixedString(text)

    def _on_log_copy(self):
        selection = self.tbv_logs.selectedIndexes()
        text = self.origin_model.get_text_by_indexes(selection)
        QApplication.clipboard().setText(text)

    def _on_vsbar_value_changed(self, value):
        vsbar = self.tbv_logs.verticalScrollBar()
        maximun = vsbar.maximum()
        self._auto_scroll_to_bottom = value == maximun
        # log.debug(f"自动滚动 {self._auto_scroll_to_bottom}")

    def on_start(self, checked):
        if checked:
            self.origin_model.start(self.device.serial)
        else:
            self.origin_model.stop()

    def _on_scroll_to_bottom(self, index, first, last):
        # 可能有性能问题，但是要观察
        self.tbv_logs.setUpdatesEnabled(False)
        for i in range(first, last):
            self.tbv_logs.resizeRowToContents(i)
        self.tbv_logs.setUpdatesEnabled(True)
        if self._auto_scroll_to_bottom:
            self.tbv_logs.scrollToBottom()

    def _on_btn_save(self):
        text = self.origin_model.log_text()
        try:
            filepath, type = QFileDialog.getSaveFileName(
                self, "文件保存", "/", "log(*.log)"
            )
            file = open(filepath, "w", encoding="utf-8")
            log.debug(f"保存的文件路径：{filepath}")
            file.writelines(text)
            file.close()
        except Exception:
            QMessageBox.critical(self, "错误", "没有指定保存的文件名")


if __name__ == "__main__":

    app = QApplication()
    win = LogcatWidget()
    win.show()
    sys.exit(app.exec())



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'ui_title_bar.ui'
##
## Created by: Qt User Interface Compiler version 6.3.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (
    QCoreApplication,
    QDate,
    QDateTime,
    QLocale,
    QMetaObject,
    QObject,
    QPoint,
    QRect,
    QSize,
    QTime,
    QUrl,
    Qt,
)
from PySide6.QtGui import (
    QBrush,
    QColor,
    QConicalGradient,
    QCursor,
    QFont,
    QFontDatabase,
    QGradient,
    QIcon,
    QImage,
    QKeySequence,
    QLinearGradient,
    QPainter,
    QPalette,
    QPixmap,
    QRadialGradient,
    QTransform,
)
from PySide6.QtWidgets import (
    QApplication,
    QFrame,
    QHBoxLayout,
    QLabel,
    QSizePolicy,
    QToolButton,
    QVBoxLayout,
    QWidget,
)
import asset_rc
import asset_rc


class Ui_TitleBar(object):
    def setupUi(self, TitleBar):
        if not TitleBar.objectName():
            TitleBar.setObjectName("TitleBar")
        TitleBar.resize(722, 40)
        TitleBar.setMinimumSize(QSize(0, 40))
        TitleBar.setMaximumSize(QSize(16777215, 40))
        TitleBar.setStyleSheet(
            "#TitleBar #bg{\n"
            "background-color:#343b48;\n"
            "border-radius:8;\n"
            "}\n"
            "\n"
            "#TitleBar #logo {\n"
            "border: 1px solid #3c4454;\n"
            "padding-right:10px;\n"
            "border-top:none;\n"
            "border-bottom:none;\n"
            "border-left:none;\n"
            "margin-right:5px;\n"
            "border-radius:0;\n"
            "}\n"
            "\n"
            "#TitleBar  #logo_title{\n"
            "color:#77b3f1;\n"
            "font-size:20px;\n"
            "font-weight:bold;\n"
            "}\n"
            "\n"
            "#TitleBar  #lb_title{\n"
            "font-size:14px;\n"
            "color:#6c7c96;\n"
            "}\n"
            "\n"
            "#TitleBar  QToolButton{\n"
            "background-color:#343b48;\n"
            "border:none;\n"
            "border-radius:4;\n"
            "padding: 4;\n"
            "}\n"
            "\n"
            "#TitleBar QToolButton:hover,\n"
            "#TitleBar QToolButton:checked{\n"
            "background-color:#3c4454;\n"
            "}\n"
            "\n"
            "\n"
            "#TitleBar QToolButton:pressed{\n"
            "background-color:#e2e9f7;\n"
            "}\n"
            "\n"
            "#TitleBar #tool{\n"
            "border:1px solid #3c4454;\n"
            "border-top:none;\n"
            "border-bottom:none;\n"
            "margin-right:5px;\n"
            "border-radius:0;\n"
            "}"
        )
        self.verticalLayout = QVBoxLayout(TitleBar)
        self.verticalLayout.setObjectName("verticalLayout")
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.bg = QFrame(TitleBar)
        self.bg.setObjectName("bg")
        self.bg.setMinimumSize(QSize(0, 40))
        self.bg.setMaximumSize(QSize(16777215, 40))
        self.bg.setFrameShape(QFrame.StyledPanel)
        self.bg.setFrameShadow(QFrame.Raised)
        self.horizontalLayout = QHBoxLayout(self.bg)
        self.horizontalLayout.setSpacing(0)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.horizontalLayout.setContentsMargins(6, 6, 6, 6)
        self.logo = QFrame(self.bg)
        self.logo.setObjectName("logo")
        self.logo.setMaximumSize(QSize(130, 16777215))
        self.logo.setFrameShape(QFrame.StyledPanel)
        self.logo.setFrameShadow(QFrame.Raised)
        self.horizontalLayout_2 = QHBoxLayout(self.logo)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.horizontalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.logo_icon = QLabel(self.logo)
        self.logo_icon.setObjectName("logo_icon")
        self.logo_icon.setPixmap(QPixmap(":/icon_b/assets/svg_blue/xing.svg"))

        self.horizontalLayout_2.addWidget(self.logo_icon)

        self.logo_title = QLabel(self.logo)
        self.logo_title.setObjectName("logo_title")

        self.horizontalLayout_2.addWidget(self.logo_title)

        self.horizontalLayout.addWidget(self.logo)

        self.lb_title = QLabel(self.bg)
        self.lb_title.setObjectName("lb_title")

        self.horizontalLayout.addWidget(self.lb_title)

        self.tool = QFrame(self.bg)
        self.tool.setObjectName("tool")
        self.tool.setMaximumSize(QSize(80, 16777215))
        self.tool.setFrameShape(QFrame.StyledPanel)
        self.tool.setFrameShadow(QFrame.Raised)
        self.horizontalLayout_3 = QHBoxLayout(self.tool)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.horizontalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.btn_search = QToolButton(self.tool)
        self.btn_search.setObjectName("btn_search")
        icon = QIcon()
        icon.addFile(
            ":/icon_w/assets/svg_white/search.svg", QSize(), QIcon.Normal, QIcon.Off
        )
        icon.addFile(
            ":/icon_b/assets/svg_blue/search.svg", QSize(), QIcon.Normal, QIcon.On
        )
        self.btn_search.setIcon(icon)

        self.horizontalLayout_3.addWidget(self.btn_search)

        self.btn_setting = QToolButton(self.tool)
        self.btn_setting.setObjectName("btn_setting")
        icon1 = QIcon()
        icon1.addFile(
            ":/icon_w/assets/svg_white/cog.svg", QSize(), QIcon.Normal, QIcon.Off
        )
        icon1.addFile(
            ":/icon_b/assets/svg_blue/cog.svg", QSize(), QIcon.Normal, QIcon.On
        )
        self.btn_setting.setIcon(icon1)
        self.btn_setting.setCheckable(True)

        self.horizontalLayout_3.addWidget(self.btn_setting)

        self.horizontalLayout.addWidget(self.tool)

        self.control_btn = QFrame(self.bg)
        self.control_btn.setObjectName("control_btn")
        self.control_btn.setMaximumSize(QSize(94, 16777215))
        self.control_btn.setFrameShape(QFrame.StyledPanel)
        self.control_btn.setFrameShadow(QFrame.Raised)
        self.horizontalLayout_4 = QHBoxLayout(self.control_btn)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.horizontalLayout_4.setContentsMargins(0, 0, 0, 0)
        self.btn_min = QToolButton(self.control_btn)
        self.btn_min.setObjectName("btn_min")
        self.btn_min.setLayoutDirection(Qt.LeftToRight)
        self.btn_min.setStyleSheet("")
        icon2 = QIcon()
        icon2.addFile(
            ":/icon_w/assets/svg_white/minus.svg", QSize(), QIcon.Normal, QIcon.Off
        )
        icon2.addFile(
            ":/icon_b/assets/svg_blue/minus.svg", QSize(), QIcon.Normal, QIcon.On
        )
        self.btn_min.setIcon(icon2)

        self.horizontalLayout_4.addWidget(self.btn_min)

        self.btn_max = QToolButton(self.control_btn)
        self.btn_max.setObjectName("btn_max")
        self.btn_max.setLayoutDirection(Qt.LeftToRight)
        self.btn_max.setStyleSheet("")
        icon3 = QIcon()
        icon3.addFile(
            ":/icon_w/assets/svg_white/checkbox-unchecked.svg",
            QSize(),
            QIcon.Normal,
            QIcon.Off,
        )
        icon3.addFile(
            ":/icon_b/assets/svg_blue/checkbox-unchecked.svg",
            QSize(),
            QIcon.Normal,
            QIcon.On,
        )
        self.btn_max.setIcon(icon3)
        self.btn_max.setCheckable(True)

        self.horizontalLayout_4.addWidget(self.btn_max)

        self.btn_close = QToolButton(self.control_btn)
        self.btn_close.setObjectName("btn_close")
        self.btn_close.setLayoutDirection(Qt.LeftToRight)
        self.btn_close.setStyleSheet("")
        icon4 = QIcon()
        icon4.addFile(
            ":/icon_w/assets/svg_white/cross.svg", QSize(), QIcon.Normal, QIcon.On
        )
        icon4.addFile(
            ":/icon_b/assets/svg_blue/cross.svg", QSize(), QIcon.Disabled, QIcon.Off
        )
        self.btn_close.setIcon(icon4)
        self.btn_close.setToolButtonStyle(Qt.ToolButtonIconOnly)

        self.horizontalLayout_4.addWidget(self.btn_close)

        self.horizontalLayout.addWidget(self.control_btn)

        self.verticalLayout.addWidget(self.bg)

        self.retranslateUi(TitleBar)

        QMetaObject.connectSlotsByName(TitleBar)

    # setupUi

    def retranslateUi(self, TitleBar):
        TitleBar.setWindowTitle(QCoreApplication.translate("TitleBar", "Form", None))
        self.logo_icon.setText("")
        self.logo_title.setText(QCoreApplication.translate("TitleBar", "Perfcat", None))
        self.lb_title.setText(
            QCoreApplication.translate(
                "TitleBar", "\u5c0f\u82f1\u96c4-xxx-\u6027\u80fd\u6d4b\u8bd5", None
            )
        )
        self.btn_search.setText("")
        self.btn_setting.setText("")
        self.btn_min.setText(QCoreApplication.translate("TitleBar", "...", None))
        self.btn_max.setText(QCoreApplication.translate("TitleBar", "...", None))
        self.btn_close.setText(QCoreApplication.translate("TitleBar", "...", None))

    # retranslateUi



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'ui_left_menu.ui'
##
## Created by: Qt User Interface Compiler version 6.3.0
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (
    QCoreApplication,
    QDate,
    QDateTime,
    QLocale,
    QMetaObject,
    QObject,
    QPoint,
    QRect,
    QSize,
    QTime,
    QUrl,
    Qt,
)
from PySide6.QtGui import (
    QBrush,
    QColor,
    QConicalGradient,
    QCursor,
    QFont,
    QFontDatabase,
    QGradient,
    QIcon,
    QImage,
    QKeySequence,
    QLinearGradient,
    QPainter,
    QPalette,
    QPixmap,
    QRadialGradient,
    QTransform,
)
from PySide6.QtWidgets import (
    QApplication,
    QFrame,
    QPushButton,
    QScrollArea,
    QSizePolicy,
    QSpacerItem,
    QVBoxLayout,
    QWidget,
)
import asset_rc


class Ui_LeftMenu(object):
    def setupUi(self, LeftMenu):
        if not LeftMenu.objectName():
            LeftMenu.setObjectName("LeftMenu")
        LeftMenu.resize(244, 732)
        sizePolicy = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(LeftMenu.sizePolicy().hasHeightForWidth())
        LeftMenu.setSizePolicy(sizePolicy)
        LeftMenu.setMinimumSize(QSize(50, 0))
        LeftMenu.setMaximumSize(QSize(244, 16777215))
        LeftMenu.setStyleSheet(
            "#LeftMenu {\n"
            "background-color: #1b1e23;\n"
            "border-radius:8;\n"
            "}\n"
            "\n"
            "QPushButton {\n"
            "background-color:#1b1e23;\n"
            "border-radius: 8;\n"
            "height:40;\n"
            "text-align:left;\n"
            "padding-left:15;\n"
            "padding-top:2;\n"
            "padding-bottom:2;\n"
            "color:#6b7884;\n"
            "}\n"
            "\n"
            "QPushButton:hover{\n"
            "background-color:#21252d;\n"
            "}\n"
            "\n"
            "QPushButton:pressed{\n"
            "background-color:#2c313c;\n"
            "}\n"
            "\n"
            "QPushButton:checked{\n"
            "background-color:#2c313c;\n"
            "}\n"
            "\n"
            "\n"
            "#bottom{\n"
            "border-top:1 solid #272c36;\n"
            "border-radius: 0;\n"
            "}\n"
            "\n"
            "#nav_menu {\n"
            "border-top:1 solid #272c36;\n"
            "border-radius: 0\uff1b\n"
            "}\n"
            "\n"
            "#LeftMenu #scrollArea,#scrollAreaWidgetContents_3{\n"
            "background-color:#1b1e23;\n"
            "border:none;\n"
            "}\n"
            "\n"
            "#btn_toggle:checked{\n"
            "background-color:#21252d;\n"
            "}\n"
            "\n"
            "\n"
            "QToolTip { \n"
            "color:#6b7884;\n"
            "background-color: #1b1e23; \n"
            "border: 0px; \n"
            "border-radius:8;\n"
            "border-left: 2px solid #4f9fee;\n"
            "}"
        )
        self.verticalLayout = QVBoxLayout(LeftMenu)
        self.verticalLayout.setSpacing(6)
        self.verticalLayout.setObjectName("verticalLayout")
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.bg = QFrame(LeftMenu)
        self.bg.setObjectName("bg")
        sizePolicy.setHeightForWidth(self.bg.sizePolicy().hasHeightForWidth())
        self.bg.setSizePolicy(sizePolicy)
        self.bg.setMinimumSize(QSize(240, 0))
        self.bg.setMaximumSize(QSize(240, 16777215))
        self.bg.setStyleSheet("")
        self.bg.setFrameShape(QFrame.StyledPanel)
        self.bg.setFrameShadow(QFrame.Raised)
        self.verticalLayout_6 = QVBoxLayout(self.bg)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.verticalLayout_6.setContentsMargins(0, 0, 0, 0)
        self.top = QFrame(self.bg)
        self.top.setObjectName("top")
        self.top.setMaximumSize(QSize(16777215, 46))
        self.top.setLayoutDirection(Qt.RightToLeft)
        self.top.setStyleSheet("")
        self.top.setFrameShape(QFrame.StyledPanel)
        self.top.setFrameShadow(QFrame.Raised)
        self.top.setLineWidth(1)
        self.verticalLayout_3 = QVBoxLayout(self.top)
        self.verticalLayout_3.setSpacing(6)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.verticalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.btn_toggle = QPushButton(self.top)
        self.btn_toggle.setObjectName("btn_toggle")
        self.btn_toggle.setLayoutDirection(Qt.LeftToRight)
        icon = QIcon()
        icon.addFile(
            ":/icon_w/assets/svg_white/menu.svg", QSize(), QIcon.Normal, QIcon.Off
        )
        icon.addFile(
            ":/icon_b/assets/svg_blue/circle-left.svg", QSize(), QIcon.Normal, QIcon.On
        )
        self.btn_toggle.setIcon(icon)
        self.btn_toggle.setIconSize(QSize(16, 16))
        self.btn_toggle.setCheckable(True)

        self.verticalLayout_3.addWidget(self.btn_toggle)

        self.verticalLayout_6.addWidget(self.top)

        self.scrollArea = QScrollArea(self.bg)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollArea.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.scrollArea.setWidgetResizable(True)
        self.scrollAreaWidgetContents_3 = QWidget()
        self.scrollAreaWidgetContents_3.setObjectName("scrollAreaWidgetContents_3")
        self.scrollAreaWidgetContents_3.setGeometry(QRect(0, 0, 238, 571))
        self.verticalLayout_2 = QVBoxLayout(self.scrollAreaWidgetContents_3)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.nav_menu = QFrame(self.scrollAreaWidgetContents_3)
        self.nav_menu.setObjectName("nav_menu")
        self.nav_menu.setLayoutDirection(Qt.LeftToRight)
        self.nav_menu.setStyleSheet("")
        self.nav_menu.setFrameShape(QFrame.StyledPanel)
        self.nav_menu.setFrameShadow(QFrame.Raised)
        self.verticalLayout_5 = QVBoxLayout(self.nav_menu)
        self.verticalLayout_5.setSpacing(6)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.verticalLayout_5.setContentsMargins(0, 5, 0, 0)
        self.btn_home = QPushButton(self.nav_menu)
        self.btn_home.setObjectName("btn_home")
        icon1 = QIcon()
        icon1.addFile(
            ":/icon_w/assets/svg_white/home.svg", QSize(), QIcon.Normal, QIcon.Off
        )
        icon1.addFile(
            ":/icon_b/assets/svg_blue/home.svg", QSize(), QIcon.Normal, QIcon.On
        )
        self.btn_home.setIcon(icon1)
        self.btn_home.setCheckable(True)

        self.verticalLayout_5.addWidget(self.btn_home)

        self.verticalLayout_2.addWidget(self.nav_menu)

        self.verticalSpacer = QSpacerItem(
            20, 494, QSizePolicy.Minimum, QSizePolicy.Expanding
        )

        self.verticalLayout_2.addItem(self.verticalSpacer)

        self.scrollArea.setWidget(self.scrollAreaWidgetContents_3)

        self.verticalLayout_6.addWidget(self.scrollArea)

        self.bottom = QFrame(self.bg)
        self.bottom.setObjectName("bottom")
        self.bottom.setStyleSheet("")
        self.bottom.setFrameShape(QFrame.StyledPanel)
        self.bottom.setFrameShadow(QFrame.Raised)
        self.verticalLayout_4 = QVBoxLayout(self.bottom)
        self.verticalLayout_4.setSpacing(6)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.verticalLayout_4.setContentsMargins(0, 6, 0, 0)
        self.btn_about = QPushButton(self.bottom)
        self.btn_about.setObjectName("btn_about")
        icon2 = QIcon()
        icon2.addFile(
            ":/icon_w/assets/svg_white/info.svg", QSize(), QIcon.Normal, QIcon.Off
        )
        icon2.addFile(
            ":/icon_b/assets/svg_blue/info.svg", QSize(), QIcon.Normal, QIcon.On
        )
        self.btn_about.setIcon(icon2)
        self.btn_about.setCheckable(True)

        self.verticalLayout_4.addWidget(self.btn_about)

        self.btn_setting = QPushButton(self.bottom)
        self.btn_setting.setObjectName("btn_setting")
        icon3 = QIcon()
        icon3.addFile(
            ":/icon_w/assets/svg_white/wrench.svg", QSize(), QIcon.Normal, QIcon.Off
        )
        icon3.addFile(
            ":/icon_b/assets/svg_blue/wrench.svg", QSize(), QIcon.Normal, QIcon.On
        )
        self.btn_setting.setIcon(icon3)
        self.btn_setting.setCheckable(True)
        self.btn_setting.setChecked(False)

        self.verticalLayout_4.addWidget(self.btn_setting)

        self.verticalLayout_6.addWidget(self.bottom)

        self.verticalLayout.addWidget(self.bg)

        self.retranslateUi(LeftMenu)

        QMetaObject.connectSlotsByName(LeftMenu)

    # setupUi

    def retranslateUi(self, LeftMenu):
        LeftMenu.setWindowTitle(QCoreApplication.translate("LeftMenu", "Form", None))
        # if QT_CONFIG(tooltip)
        self.btn_toggle.setToolTip(
            QCoreApplication.translate(
                "LeftMenu",
                "<html><head/><body><p>\u9690\u85cf\u83dc\u5355</p></body></html>",
                None,
            )
        )
        # endif // QT_CONFIG(tooltip)
        # if QT_CONFIG(whatsthis)
        self.btn_toggle.setWhatsThis(
            QCoreApplication.translate(
                "LeftMenu", "<html><head/><body><p><br/></p></body></html>", None
            )
        )
        # endif // QT_CONFIG(whatsthis)
        self.btn_toggle.setText(
            QCoreApplication.translate("LeftMenu", "    \u9690\u85cf\u83dc\u5355", None)
        )
        self.btn_home.setText(
            QCoreApplication.translate("LeftMenu", "    \u9996\u9875", None)
        )
        # if QT_CONFIG(tooltip)
        self.btn_about.setToolTip(
            QCoreApplication.translate("LeftMenu", "\u5173\u4e8e", None)
        )
        # endif // QT_CONFIG(tooltip)
        self.btn_about.setText(
            QCoreApplication.translate("LeftMenu", "    \u5173\u4e8e", None)
        )
        # if QT_CONFIG(tooltip)
        self.btn_setting.setToolTip(
            QCoreApplication.translate("LeftMenu", "\u8bbe\u7f6e", None)
        )
        # endif // QT_CONFIG(tooltip)
        self.btn_setting.setText(
            QCoreApplication.translate("LeftMenu", "    \u8bbe\u7f6e", None)
        )

    # retranslateUi



# here put the import lib
from ppadb.device import Device
from .base.chart import MonitorChart
from perfcat.modules.profiler.fps import FpsSampler
from PySide6.QtCharts import QLineSeries

class FrameTimeMonitor(MonitorChart):
    def __init__(self, parent):
        super().__init__(
            parent,
            y_axis_name="FrameTime",
        )
        self.setObjectName("FrameTime")
        self.fps_sampler = None

        self._sample_data = {}

        self.create_series("FrameTime", QLineSeries(self), lambda v:f"{v}ms")

    def sample(self, sec: int, device: Device, package_name: str):
        if self.fps_sampler is None:
            self.fps_sampler = FpsSampler(device, package_name)

        data = self.fps_sampler.data
        frametimes = data["*frametimes"]
        frametime = sum(frametimes)/len(frametimes)

        self.add_point("FrameTime", sec, frametime)

    def reset_series_data(self):
        self.fps_sampler = None
        self._sample_data = {}
        return super().reset_series_data()

    def to_dict(self, all: bool = True) -> dict:
        if all:
            return self._sample_data
        else:
            start = self.record_range[0]
            end = self.record_range[1]

            data = {}
            for k,v in self._sample_data.items():
                if start <= k <= end:
                    data[k] = v
            return data

    def from_dict(self, data: dict):
        for sec, data_table in data.items():
            frametimes = data_table["*frametimes"]
            frametime = sum(frametimes)/len(frametimes)
            self.add_point("FrameTime", int(sec), frametime)
            
            self.flush()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :   __init__.py
@Time    :   2022/04/27 16:11:12
@Author  :   Calros Teng 
@Version :   1.0
@Contact :   303359166@qq.com
@License :   (C)Copyright 2017-2018, Xin Yuan Studio
@Desc    :   主体窗口

#todo:
[1] windows操作系统的窗口动画对无边窗口不起作用，所以放大缩小没有动画，也不能贴边触发分屏
目前还不知道怎么解决这个问题，用了win32api结果抽了
"""

# here put the import lib
import logging
import PySide6
import markdown
import textwrap
from unittest.case import doModuleCleanups

from PySide6.QtWidgets import QMainWindow, QPushButton, QWidget
from PySide6.QtCore import Qt, QPoint, QRect
from PySide6.QtGui import QMouseEvent, QHoverEvent

from perfcat.pages.home.home import Page
from perfcat.settings import settings

from . import util
from .ui_mainwindow import Ui_MainWindow

from .left_menu import LeftMenu
from .title_bar import TitleBar
from .left_column import LeftColumn


log = logging.getLogger(__name__)


class MainWindow(QMainWindow, Ui_MainWindow):

    BORDER_LIMIT = 10

    LEFT_COLUMN_MAXWIDTH = 240
    LEFT_COLUMN_MINWIDTH = 0

    CONTENT_RIGHT_MAXWIDTH = 240
    CONTENT_RIGHT_MINWIDTH = 0

    def __init__(self) -> None:
        super().__init__()
        # 初始化ui文件
        self.setupUi(self)
        self.setStyleSheet("")  # 清空qt designer里写死的样式

        # 设置flag
        # 隐藏窗体边框
        self.setWindowFlags(Qt.FramelessWindowHint)
        self.setAttribute(Qt.WA_TranslucentBackground)

        # 缩放
        # todo
        self.installEventFilter(self)
        self._resizing = False
        self._resize_mode = None

        # 设置阴影（很淡……几乎看不见，有点感觉就行）
        util.set_shadow_effect(self)
        util.set_shadow_effect(self.setting_frame)

        # 清除掉page_stacked默认的widget
        self.page_stacked.removeWidget(self.page)
        self.page.deleteLater()  # 销毁掉

        # 添加左导航菜单
        self._setup_leftmenu()

        # 添加标题栏
        self._setup_title_bar()

        # 添加状态栏
        self._setup_status_bar()

        # 添加左面板栏
        self._setup_left_column()

    # region 页面相关
    def add_page(self, page: Page):
        """
        添加页面

        框架默认把页面的object_name和导航按钮的object_name设置成一样
        通过这种方式关联按钮和页面的切换

        Args:
            page (Page): _description_
        """
        page.setParent(self)

        icon = page.windowIcon()
        title = page.windowTitle()

        # warning：addwidget必须比add_nav_menu先调用，因为如果是第一个菜单还需要触发切换到第一个页面
        self.page_stacked.addWidget(page)

        self.left_menu.add_nav_menu(icon, title, page.objectName())

        log.debug(f"添加页面 page_name:{page.objectName()} title:{title}")

    def switch_page(self, page: Page):
        log.debug(f"切换页面到 {page.objectName()}")
        self.expand_setting_frame(False)  # 收起setting
        self.page_stacked.setCurrentWidget(page)
        # page.show()
        self.title_bar.btn_setting.setEnabled(page.setting_widget != None)

        if page.setting_widget:
            log.debug(f"页面 {page.objectName()} 有设置界面，设置 setting {page.setting_widget}")
            util.clear_layout(self.setting_container)
            # setting_widget 从setting_container里删除后parent引用失效，所以重新设置回page。
            page.setting_widget.setParent(page)
            self.setting_container.layout().addWidget(page.setting_widget)

    def _on_switch_page(self, button: QPushButton, checked):
        log.debug(f"触发切换页面 {button} {checked}")
        page_name = button.objectName()
        page: Page = self.page_stacked.findChild(Page, page_name)
        self.switch_page(page)

    # endregion

    @property
    def current_page(self) -> Page:
        return self.page_stacked.currentWidget()

    def set_about_info(self, text: str):
        """
        设置关于信息，支持markdown

        Args:
            text (str): 文本， 可以用markdown
        """
        text = textwrap.dedent(text)  # 对齐缩进
        html = markdown.markdown((text))
        self.left_column.te_about.setHtml(html)

    def setWindowFilePath(self, filePath: str) -> None:
        super().setWindowFilePath(filePath)
        self.windowTitleChanged.emit(self.windowTitle())

    def setWindowModified(self, arg__1: bool) -> None:
        super().setWindowModified(arg__1)
        self.windowTitleChanged.emit(self.windowTitle())

    def _setup_leftmenu(self):
        self.left_menu = LeftMenu(self)
        self.left_menu.setObjectName("LeftMenu")
        self.left_menu_frame.layout().addWidget(self.left_menu)

        # 添加阴影
        util.set_shadow_effect(self.left_menu_frame)

        # 导航按钮组点击的时候切换页面
        self.left_menu.nav_menu_group.buttonToggled.connect(self._on_switch_page)

        # 底部按钮组点击的时候触发展开左栏
        self.left_menu.bottom_btn_group.idToggled.connect(
            self._toggle_left_column_frame
        )

    def _setup_title_bar(self):
        # 添加标题栏
        self.title_bar = TitleBar(self)
        self.title_bar.setObjectName("TitleBar")
        self.title_bar_frame.layout().addWidget(self.title_bar)

        # 设置阴影
        util.set_shadow_effect(self.title_bar_frame)

        # 页面设置按钮
        self.title_bar.btn_setting.toggled.connect(self.expand_setting_frame)

    def _setup_status_bar(self):
        util.set_shadow_effect(self.status_bar_frame)

    def _setup_left_column(self):
        self.left_column = LeftColumn(self)
        self.left_column_frame.layout().addWidget(self.left_column)

        self.left_column.btn_close.clicked.connect(
            lambda: self.expand_left_column_frame(False)
        )

    @property
    def left_column_visible(self) -> bool:
        return self.left_column_frame.maximumWidth() > 0

    @property
    def setting_frame_visible(self) -> bool:
        return self.setting_frame.maximumWidth() > 0

    def _toggle_left_column_frame(self, id, checked: bool):

        if not self.left_column_visible:
            self.expand_left_column_frame(True)

        self.left_column.stacked.setCurrentIndex(id)
        button = self.left_menu.bottom_btn_group.button(id)
        button_text = button.text()
        self.left_column.title.setText(button_text)

    def expand_left_column_frame(self, checked: bool):
        util.set_h_expand_anim(
            self.left_column_frame,
            checked,
            self.LEFT_COLUMN_MAXWIDTH,
            self.LEFT_COLUMN_MINWIDTH,
        )
        if not checked:
            # 折叠回去了，清除掉底部互斥选项卡的选中
            self.left_menu.bottom_btn_group_reset()

    def expand_setting_frame(self, checked: bool):
        """
        展开当前页面的设置界面

        这个方法不仅仅用于信号调用，直接调用也可以展开和收起setting

        Args:
            checked (bool): _description_
        """
        util.set_h_expand_anim(
            self.setting_frame,
            checked,
            self.CONTENT_RIGHT_MAXWIDTH,
            self.CONTENT_RIGHT_MINWIDTH,
        )
        self.title_bar.btn_setting.setChecked(checked)

    def closeEvent(self, event: PySide6.QtGui.QCloseEvent) -> None:
        settings.setValue("mainwindow/geometry", self.saveGeometry())
        settings.setValue("mainwindow/state", self.saveState())
        return super().closeEvent(event)

    def showEvent(self, event: PySide6.QtGui.QShowEvent) -> None:
        geometry = settings.value("mainwindow/geometry", None)
        state = settings.value("mainwindow/state", None)
        if geometry and state:
            self.restoreGeometry(geometry)
            self.restoreState(state)
        return super().showEvent(event)

    def eventFilter(
        self, watched: PySide6.QtCore.QObject, event: PySide6.QtCore.QEvent
    ) -> bool:
        if isinstance(event, QHoverEvent) and not self._resizing:

            mouse_pos = event.pos()
            rect = self.rect()
            top_area = QRect(rect)
            top_area.setHeight(self.BORDER_LIMIT)

            bottom_area = QRect(rect)
            bottom_area.setTop(rect.top() + rect.height() - self.BORDER_LIMIT)
            bottom_area.setHeight(self.BORDER_LIMIT)

            left_area = QRect(rect)
            left_area.setWidth(self.BORDER_LIMIT)

            right_area = QRect(rect)
            right_area.setLeft(rect.left() + rect.width() - self.BORDER_LIMIT)

            on_top = top_area.contains(mouse_pos)
            on_bottom = bottom_area.contains(mouse_pos)
            on_left = left_area.contains(mouse_pos)
            on_right = right_area.contains(mouse_pos)
            on_lefttop_corner = on_top and on_left
            on_righttop_corner = on_top and on_right
            on_leftbottom_corner = on_left and on_bottom
            on_rightbottom_corner = on_right and on_bottom

            # print(on_top, on_bottom, on_left, on_right)
            if on_lefttop_corner:
                self.setCursor(Qt.CursorShape.SizeFDiagCursor)
                self._resize_mode = Qt.LeftEdge | Qt.TopEdge
            elif on_righttop_corner:
                self.setCursor(Qt.CursorShape.SizeBDiagCursor)
                self._resize_mode = Qt.RightEdge | Qt.TopEdge
            elif on_leftbottom_corner:
                self.setCursor(Qt.CursorShape.SizeBDiagCursor)
                self._resize_mode = Qt.LeftEdge | Qt.BottomEdge
            elif on_rightbottom_corner:
                self.setCursor(Qt.CursorShape.SizeFDiagCursor)
                self._resize_mode = Qt.RightEdge | Qt.BottomEdge
            elif on_top or on_bottom:
                self.setCursor(Qt.SizeVerCursor)
                if on_top:
                    self._resize_mode = Qt.TopEdge
                elif on_bottom:
                    self._resize_mode = Qt.BottomEdge
            elif on_left or on_right:
                self.setCursor(Qt.SizeHorCursor)
                if on_left:
                    self._resize_mode = Qt.LeftEdge
                elif on_right:
                    self._resize_mode = Qt.RightEdge
            else:
                self.setCursor(Qt.ArrowCursor)
                self._resize_mode = None

        return super().eventFilter(watched, event)

    def mousePressEvent(self, event: PySide6.QtGui.QMouseEvent) -> None:

        self._resizing = True
        return super().mousePressEvent(event)

    def mouseReleaseEvent(self, event: PySide6.QtGui.QMouseEvent) -> None:
        self._resizing = False
        return super().mouseReleaseEvent(event)

    def mouseMoveEvent(self, event: QMouseEvent) -> None:

        if self._resizing:

            if self._resize_mode:
                global_pos = event.globalPos()
                geometry = self.geometry()
                if self._resize_mode == Qt.TopEdge:
                    geometry.setTop(global_pos.y())
                elif self._resize_mode == Qt.BottomEdge:
                    geometry.setBottom(global_pos.y())
                elif self._resize_mode == Qt.LeftEdge:
                    geometry.setLeft(global_pos.x())
                elif self._resize_mode == Qt.RightEdge:
                    geometry.setRight(global_pos.x())
                elif self._resize_mode == Qt.TopEdge | Qt.LeftEdge:
                    geometry.setTopLeft(global_pos)
                elif self._resize_mode == Qt.TopEdge | Qt.RightEdge:
                    geometry.setTopRight(global_pos)
                elif self._resize_mode == Qt.BottomEdge | Qt.LeftEdge:
                    geometry.setBottomLeft(global_pos)
                elif self._resize_mode == Qt.BottomEdge | Qt.RightEdge:
                    geometry.setBottomRight(global_pos)

                if (
                    geometry.width() > self.minimumWidth()
                    and geometry.height() > self.minimumHeight()
                ):
                    self.setGeometry(geometry)

        return super().mouseMoveEvent(event)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :   profiler.py
@Time    :   2022/05/05 20:16:06
@Author  :   Calros Teng 
@Version :   1.0
@Contact :   303359166@qq.com
@License :   (C)Copyright 2017-2018, Xin Yuan Studio
@Desc    :  
todo:
[1] 如果需要设备选择和app选择体验更好点就要重新实现model和itemdelegate了。目前暂时这样吧。
[2] 最好还是开一个线程去采集性能数据，不然会阻塞主线程。

"""

# here put the import lib


import PySide6
import logging
import csv
import time
import json
import io
import os
import subprocess
import pkg_resources
from typing import Dict, List
from shutil import which

from perfcat.pages.profiler.plugins.base import MonitorChart
from . import plugins  # noqa
from ppadb.client import Client as adb
from ppadb.device import Device
from PySide6.QtWidgets import (
    QCompleter,
    QTableWidgetItem,
    QApplication,
    QMessageBox,
    QFileDialog,
)
from PySide6.QtCore import (
    Qt,
    Signal,
    QRunnable,
    SignalInstance,
    QThread,
    QElapsedTimer,
    QThreadPool,
)
from perfcat.modules.hot_plug import HotPlugWatcher
from perfcat.modules.reporter import export
from perfcat.settings import settings

from ...ui.constant import ButtonStyle
from ...ui.page import Page
from .plugins import register
from .ui_profiler import Ui_Profiler
from ...modules.profiler.device import device_info
from .logcat2 import LogcatWidget

log = logging.getLogger(__name__)


class Worker(QRunnable):
    def __init__(self, func, *args, **kwargs) -> None:
        super().__init__()
        self.func = func
        self.args = args
        self.kwargs = kwargs

    def run(self):
        self.func(*self.args, **self.kwargs)


class Profiler(Page, Ui_Profiler):
    # 当系统设备插拔的时候发出信号
    device_changed: SignalInstance = Signal()  # type:ignore
    # 初始化adb-server信号
    adb_server_starting: SignalInstance = Signal()  # type:ignore

    def __init__(self, parent) -> None:
        super().__init__(parent)

        self.setupUi(self)
        self.clear_stylesheet()

        # 让监视器layout顶部对齐（designer里无法做到只能代码设置）
        self.verticalLayout_6.setAlignment(Qt.AlignmentFlag.AlignTop)

        self.adb = adb()
        self._device_info = {}

        self.plugins: List[MonitorChart] = []

        self.sample_thread: QThread = QThread(self)
        self.tick_count = 0
        self.record_range = [0, 0]

        self.btn_connect.toggled.connect(self._connect_device)

        # 默认是没选中任何设备和app，此时连接和录制置灰
        self.btn_connect.setEnabled(False)
        self.btn_record.setEnabled(False)

        # 复制设备信息
        self.btn_copy_info.clicked.connect(self._copy_info)

        # 日志logcat模块
        self.logcat = LogcatWidget(self)
        self.verticalLayout_8.addWidget(self.logcat)

        # 当设备选择改变的时候更新连接按钮状态
        self.cbx_device.currentIndexChanged.connect(self._update_btn_status)  # 设备切换时更新
        self.cbx_device.currentIndexChanged.connect(
            self._update_device_info
        )  # 读取显示设备信息
        self.cbx_app.currentIndexChanged.connect(self._update_btn_status)  # app切换时更新
        self.cbx_app.editTextChanged.connect(self._update_btn_status)  # app名修改的时候更新

        self.cbx_device.currentIndexChanged.connect(self._update_app_list)

        # 按钮处理
        self.btn_save.clicked.connect(self._save_file)
        self.btn_open.clicked.connect(self._open_file)
        self.btn_record.toggled.connect(self._on_toggled_record)

        self._init_plugins()
        self._update_btn_status()

        self.adb_server_starting.connect(
            lambda: self.notify("adb server 启动中...", ButtonStyle.warning)
        )

        def on_splitter_clicked(event):
            self.frame_3.setHidden(not self.frame_3.isHidden())

        self.right.mouseDoubleClickEvent = on_splitter_clicked

    def _init_plugins(self):
        self.reset_h_scrollbar()
        for plugin_cls in register:
            plugin: MonitorChart = plugin_cls(self)
            self.scrollAreaWidgetContents.layout().addWidget(plugin)

            self.plugins.append(plugin)

            self.horizontalScrollBar.valueChanged.connect(plugin.set_x_offset)  # 水平滚动同步
            plugin.axis_range_size_changed.connect(self._sync_plugin_range_size)  # 缩放同步
            plugin.mark_line_changed.connect(self._sync_mark_line)  # 标线位置同步
            plugin.x_max_offset_changed.connect(self._sync_scroll_max)  # 滚动条同步最大x轴

    def start_adb_server(self):
        path = which("adb")
        default_adb_path = pkg_resources.resource_filename("perfcat", "adb/adb.exe")
        log.debug(f"系统adb:{path} 内置adb:{default_adb_path}")
        path = path or default_adb_path
        try:
            self.adb.version()
        except RuntimeError:
            self.adb_server_starting.emit()
            log.debug(f"启动adb命令:{[path, 'start-server']}")
            subprocess.call([path, "start-server"], shell=True)
            log.debug(f"测试adb server:{self.adb.version()}")

    def reset_h_scrollbar(self):
        self.horizontalScrollBar.setMaximum(0)
        self.horizontalScrollBar.setValue(0)

    def _sync_mark_line(self, pos):
        sender: MonitorChart = self.sender()
        for p in self.plugins:
            if p == sender:
                continue
            p.mark_line = sender.mark_line

    def _sync_plugin_range_size(self):
        sender: MonitorChart = self.sender()
        axis_range_size = sender.axis_range_size
        for p in self.plugins:
            if p == sender:
                continue
            p.axis_range_size = axis_range_size

    def _sync_scroll_max(self, value):
        if self.horizontalScrollBar.value() == self.horizontalScrollBar.maximum():
            self.horizontalScrollBar.setMaximum(value)
            self.horizontalScrollBar.setValue(value)
        else:
            self.horizontalScrollBar.setMaximum(value)

    def clear_all_data(self):
        for p in self.plugins:
            p.reset_series_data()

        self.reset_h_scrollbar()

    @property
    def current_device(self) -> Device:
        """
        返回当前选中的设备

        Returns:
            Device: _description_
        """
        return self.cbx_device.currentData(Qt.UserRole)

    @property
    def device_info(self) -> Dict[str, str]:
        if self.current_device is None:
            return {}

        return self._device_info

    def _update_btn_status(self):
        valid_device = self.cbx_device.currentIndex() > -1
        valid_app = (
            self.cbx_app.currentIndex() > -1 and self.cbx_app.currentText() != ""
        )

        log.debug(f"更新按钮状态 valid_device:{valid_device} valid_app:{valid_app}")
        self.btn_connect.setEnabled(valid_device and valid_app)
        # self.btn_save.setEnabled(valid_device and valid_app)

        self.logcat.device = self.current_device

    def _copy_info(self):
        """
        把设备信息复制到剪贴板
        """
        if self.current_device:
            device_info = self._device_info
            clipboard = QApplication.clipboard()
            output = io.StringIO()
            writer = csv.writer(output, csv.get_dialect("excel-tab"))
            for k, v in device_info.items():
                writer.writerow([k, v])
            text = output.getvalue()
            clipboard.setText(text)
            self.notify("复制设备信息到剪贴板", ButtonStyle.success)

    def _update_device_info(self):
        """
        更新设备信息

        _extended_summary_
        """
        thread = QThread(self)

        def run():
            # 缓存到成员变量里免得每次都重新获取阻塞UI
            if self.current_device:
                dev_info = self._device_info = device_info(self.current_device)
            else:
                dev_info = self._device_info = {}

            # 清空数据
            self.tb_device_info.clearContents()

            if not dev_info:
                self.btn_copy_info.setEnabled(False)
                return

            # 设置行数
            self.tb_device_info.setRowCount(len(dev_info))

            # 填写数据
            index = 0
            for prop, value in dev_info.items():
                prop_item = QTableWidgetItem(prop)
                prop_item.setFlags(prop_item.flags() ^ Qt.ItemIsEditable)
                self.tb_device_info.setItem(index, 0, prop_item)
                value_item = QTableWidgetItem(value)
                value_item.setToolTip(value)
                value_item.setFlags(prop_item.flags() ^ Qt.ItemIsEditable)
                self.tb_device_info.setItem(index, 1, value_item)
                index += 1
            self.btn_copy_info.setEnabled(True)

        thread.run = run
        thread.start()

    def _update_app_list(self, index: int):
        if not self.current_device:
            self.cbx_app.clear()
            return
        packages = self.current_device.list_packages()
        self.cbx_app.clear()
        items = packages
        self.cbx_app.addItems(items)
        completer = QCompleter(items, self.cbx_app)
        completer.setCaseSensitivity(Qt.CaseInsensitive)
        completer.setFilterMode(Qt.MatchContains)
        self.cbx_app.setCompleter(completer)

    def _update_devices_list(self):
        devices: List[Device] = self.adb.devices(state="device")
        pre_selected = self.cbx_device.currentText()
        self.cbx_device.clear()
        for dev in devices:
            model = dev.get_properties()["ro.product.model"]
            name = dev.get_properties()["ro.product.name"]
            self.cbx_device.addItem(f"[{dev.serial}] {model}-{name}", dev)

        self.cbx_device.setCurrentText(pre_selected)
        # 这一段检查当前选中的设备是不是断开了
        if self.cbx_device.currentIndex() == -1 and pre_selected != "":
            self.notify("当前选择的设备已经断开！", ButtonStyle.danger)
            # 如果断开了不管是不是正在连接都强制断开
            self._connect_device(False)

    def showEvent(self, event: PySide6.QtGui.QShowEvent) -> None:
        # 这种神奇的用法……用monkeypatch的方式强行把run方法替换成内部函数，就不用派生了
        # 这样可以快速的编写异步代码，防止阻塞UI线程
        # 如果不需要维护和释放thread，那么就不需要self里声明一个变量来保存了
        # 这样写用完就抛，python会帮我们释放这个thread对象
        thread = QThread(self)

        def run():
            HotPlugWatcher.device_added.connect(self._on_device_add)
            HotPlugWatcher.device_removed.connect(self._on_device_removed)
            self.start_adb_server()
            # self.devices = set(self.adb.devices(state="device"))
            self._update_devices_list()
            # log.debug(f"刷新设备列表 {self.devices}")

        thread.run = run
        thread.start()

        return super().showEvent(event)

    def hideEvent(self, event: PySide6.QtGui.QHideEvent) -> None:
        HotPlugWatcher.device_added.disconnect(self._on_device_add)
        HotPlugWatcher.device_removed.disconnect(self._on_device_removed)

        self._connect_device(False)
        return super().hideEvent(event)

    def _on_device_add(self):
        devices: List[Device] = self.adb.devices(state="device")
        count = len(devices)
        if self.cbx_device.count() != count:
            self.notify("发现新设备！", ButtonStyle.success)
            # todo: 添加新设备item
            self._update_devices_list()

    def _on_device_removed(self):
        devices: List[Device] = self.adb.devices(state="device")
        count = len(devices)
        if self.cbx_device.count() != count:
            self.notify("设备被移除！", ButtonStyle.warning)
            # todo: 移除旧设备item，如果旧设备的serial正好是当前连接中设备，那么就置空currentIndex
            self._update_devices_list()

    def timerEvent(self, event: PySide6.QtCore.QTimerEvent) -> None:
        return super().timerEvent(event)

    def start_tick(self):
        log.debug("开始采样")
        self.tick_count = 0

        def _run():
            # 先采样缓存采样点，然后再统一刷入可以保证多个monitor同步刷新
            # 统一采样
            pool = QThreadPool.globalInstance()
            while not self.sample_thread.isInterruptionRequested():
                # 记录这次采样耗时
                time_counter = QElapsedTimer()
                time_counter.start()

                for p in self.plugins:
                    # 用线程池来启动每一个monitor的采样方法
                    # 测试发现 adb的每次命令调用耗时大概0.01ms
                    # 如果采样的实现方式有多次命令调用，那么采样耗时会累积的很高
                    # 如果每个monitor的采样采用遍历切同步的方式去处理就会很容易累计超过1s采样间隔
                    # 导致采样间隔不固定
                    # 现在改为用线程池开worker去并发处理每个monitor的sample
                    # 然后线程池等待所有worker结束再进入下一轮采样
                    workder = Worker(
                        p.sample,
                        self.tick_count,
                        self.current_device,
                        self.cbx_app.currentText(),
                    )

                    pool.start(workder)

                pool.waitForDone()

                # 统一刷入系列，让图标绘制折线
                for p in self.plugins:
                    p.flush()

                # 获取采样耗时
                take_sec = time_counter.elapsed()
                # 如果采样时间不足1s，那么剩下空闲时间让采样tick线程把这段时间睡过去
                if take_sec < 1000:
                    self.sample_thread.msleep(1000 - take_sec)
                self.tick_count += 1
            log.debug("采样线程退出")

        self.sample_thread.run = _run
        self.sample_thread.start()

    def stop_tick(self):
        log.debug("停止采样")
        if self.sample_thread.isRunning():
            self.sample_thread.requestInterruption()  # 中断线程
            self.sample_thread.wait()  # 主线程同步等待线程退出

    def destroy(self, destroyWindow: bool = ..., destroySubWindows: bool = ...) -> None:
        self.stop_tick()
        return super().destroy(destroyWindow, destroySubWindows)

    def _connect_device(self, enable: bool = True):
        """
        连接设备

        这个方法说是连接设备，其实只是开启tick定时器来轮询设备而已

        Args:
            enable (bool, optional): True开始轮询，False结束轮询. Defaults to True.
        """

        if enable:
            log.debug(f"连接设备 {self.current_device.serial}")
            self.notify(f"连接设备 {self.current_device.serial}", ButtonStyle.success)
            self.clear_all_data()
            self.start_tick()
            self.btn_open.setEnabled(False)
            self.btn_save.setEnabled(True)
            self.record_range = [0, 0]
        else:
            if self.current_device:  # current_device非none就是还连着usb
                log.debug(f"断开设备 {self.current_device.serial}")
                self.notify(f"断开设备 {self.current_device.serial}", ButtonStyle.warning)

            self.stop_tick()
            self.btn_record.setChecked(False)
            self.btn_open.setEnabled(True)
            self.btn_save.setEnabled(self.tick_count != 0)

        self.cbx_device.setDisabled(enable)
        self.cbx_app.setDisabled(enable)

        # 先拦截信号防止setchecked的时候发出toggled信号导致执行两次
        self.btn_connect.blockSignals(True)
        self.btn_connect.setChecked(enable)
        self.btn_connect.blockSignals(False)

        self.btn_record.setEnabled(enable)

    def _save_file(self):
        device_name = self._device_info["型号"]
        app_name = self.cbx_app.currentText()

        result = QMessageBox.question(
            self,
            "保存全部?",
            "是:保存全部数据\n否:仅保存录制的那部分",
            QMessageBox.Yes | QMessageBox.No | QMessageBox.Cancel,
            QMessageBox.Yes,
        )
        if result == QMessageBox.Cancel:
            return

        if result == QMessageBox.Yes:
            data = self._get_data()
        elif result == QMessageBox.No:
            data = self._get_data(False)

        date_str = time.strftime("%Y-%m-%d_%H-%M-%S")
        last_dir = settings.value("profiler/last_dir", "")

        file_names = QFileDialog.getSaveFileName(
            self,
            "保存记录",
            os.path.join(last_dir, f"{device_name}_{app_name}_{date_str}"),
            "all(*.*);;perfcat(*.pc);;excel(*.xlsx)",
        )
        if file_names[0]:
            if file_names[1] in ["perfcat(*.pc)", "all(*.*)"]:
                self.btn_record.setChecked(False)
                self.update()
                file_name = (
                    file_names[0]
                    if not os.path.splitext(file_names[0])
                    else file_names[0] + ".pc"
                )
                with open(file_name, "w", encoding="utf-8") as f:
                    json.dump(data, f, ensure_ascii=False, indent=4)
                    self.notify(f"保存到 {file_name}", ButtonStyle.success)
                last_dir = os.path.dirname(file_names[0])
                settings.setValue("profiler/last_dir", last_dir)

            if file_names[1] in ["excel(*.xlsx)", "all(*.*)"]:
                file_name = (
                    file_names[0]
                    if not os.path.splitext(file_names[0])
                    else file_names[0] + ".xlsx"
                )
                export(app_name, data, file_name)

    def _open_file(self):
        last_dir = settings.value("profiler/last_dir", "")
        file_name = QFileDialog.getOpenFileName(self, "保存记录", last_dir, "perfcat(*.pc)")

        if file_name[0]:
            with open(file_name[0], "r", encoding="utf-8") as f:
                data = json.load(f)
                data = data["data"]

                self.clear_all_data()

                for plugin in self.plugins:
                    if plugin.objectName() in data:
                        plugin.from_dict(data[plugin.objectName()])

                self.notify(f"打开:{file_name[0]}", ButtonStyle.success)

    def _get_data(self, all: bool = True):
        data = {"data": {}}
        for plugin in self.plugins:
            if all:
                _p_data = plugin.to_dict()
            elif not all:
                # todo: 根据record切割数据
                _p_data = plugin.to_dict(False)

            data["data"][plugin.objectName()] = _p_data
        data["device_info"] = self.device_info
        data["tick_count"] = self.tick_count
        if all:
            data["record_range"] = (0, self.tick_count)
        else:
            data["record_range"] = self.record_range

        return data

    def _on_toggled_record(self, checked: bool):
        if checked:
            self.record_range[0] = self.tick_count
            self.notify("开始记录", ButtonStyle.info)
            # 遍历插件开启记录线绘制（就是画条线来显示从哪到哪是记录的）
            # 记录的截取则是在这里加工处理，插件不负责截取区间数据
            for p in self.plugins:
                p.record_enable(True, self.tick_count)
        else:
            self.record_range[1] = self.tick_count
            # 遍历插件关闭记录线绘制（就是画条线来显示从哪到哪是记录的）
            for p in self.plugins:
                p.record_enable(False)
            log.debug(f"结束录制，录制的时间范围是 {self.record_range}")
            self.notify("结束记录", ButtonStyle.info)
            # todo: 结束记录后要做什么，比如直接复制剪贴板？保存报告？可以在这后面处理



from typing import List
from .base import MonitorChart
from .cpu_monitor import CpuMonitor
from .mem_monitor import MemMonitor
from .temp_monitor import TempMonitor
from .fps_monitor import FpsMonitor
from .frametime_monitor import FrameTimeMonitor

register: List[MonitorChart] = [
    FpsMonitor,
    CpuMonitor,
    MemMonitor,
    FrameTimeMonitor,
    TempMonitor,
]



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :   __init__.py
@Time    :   2022/04/29 12:12:58
@Author  :   Calros Teng 
@Version :   1.0
@Contact :   303359166@qq.com
@License :   (C)Copyright 2017-2018, Xin Yuan Studio
@Desc    :   __version__ 会作为包版本
"""

# here put the import lib

# 把asset/放到搜索目录里，这样.ui文件里 import asset_rc 才能正常导入asset_rc
import logging
import sys
import pkg_resources
from . import logger
from email import message_from_string
from importlib.metadata import metadata
from webob.multidict import MultiDict

log = logging.getLogger(__name__)

asset_dir = pkg_resources.resource_filename(__package__, ".")
sys.path.append(asset_dir)


# 读取和设置包的元信息
__version__ = "1.0.7"
__author__ = "kaluluosi"
__author_email__ = "kaluluosi@gmail.com"
__meta__ = {}

try:
    # todo: nuitka运行时无法获取到perfcat这个包的元数据，不知道以后怎么解决
    METADATA = pkg_resources.get_distribution(__package__).get_metadata("METADATA")
    msg = message_from_string(METADATA)
    __meta__ = MultiDict(msg)
    __version__ = __meta__["Version"]
    __author__ = __meta__["Author"]
    __author_email__ = __meta__["Author-email"]
except Exception as e:
    log.exception(e)
